"""
Unit tests for stem.descriptor.bandwidth_file.
"""

import datetime
import unittest

import stem.descriptor

from stem.descriptor.bandwidth_file import BandwidthFile
from test.unit.descriptor import get_resource

try:
  # added in python 2.7
  from collections import OrderedDict
except ImportError:
  from stem.util.ordereddict import OrderedDict

try:
  # added in python 3.3
  from unittest.mock import Mock, patch
except ImportError:
  from mock import Mock, patch

EXPECTED_MEASUREMENT_1 = {
  'scanner': '/scanner.1/scan-data/bws-0.0:0.8-done-2019-01-13-22:55:22',
  'measured_at': '1547441722',
  'pid_delta': '1.07534299311',
  'updated_at': '1547441722',
  'pid_error_sum': '3.23746667827',
  'nick': 'baldr',
  'node_id': '$D8B9CAA5B818DEFE80857F83FDABBB6429DCFCA0',
  'pid_bw': '47625769',
  'bw': '47600',
  'pid_error': '3.23746667827',
  'circ_fail': '0.0',
}

EXPECTED_MEASUREMENT_2 = {
  'desc_bw_obs_last': '473188',
  'success': '13',
  'desc_bw_obs_mean': '581671',
  'bw_median': '202438',
  'nick': 'Teinetteiine',
  'bw': '1',
  'desc_bw_avg': '1024000',
  'time': '2019-01-13T12:21:29',
  'bw_mean': '184647',
  'error_circ': '0',
  'error_stream': '0',
  'node_id': '$9C7E1AFDACC53228F6FB57B3A08C7D36240B8F6F',
  'error_misc': '0',
}

EXPECTED_MEASUREMENT_3 = {
  'desc_bw_obs_last': '70423',
  'desc_bw_obs_mean': '81784',
  'bw_median': '2603',
  'nick': 'whitsun',
  'bw': '1',
  'desc_bw_avg': '1073741824',
  'time': '2019-04-21T10:22:16',
  'bw_mean': '2714',
  'error_circ': '1',
  'error_stream': '0',
  'node_id': '$8F0F49F2341C7F706D5B475815DBD3E5761334B3',
  'error_misc': '0',
  'consensus_bandwidth': '1000',
  'consensus_bandwidth_is_unmeasured': 'False',
  'desc_bw_bur': '1073741824',
  'error_destination': '0',
  'error_second_relay': '0',
  'master_key_ed25519': 'acShTw35dmVSTkhMdmo9RFRLsP4QV+qOZrEJQubnvWY',
  'relay_in_recent_consensus_count': '22',
  'relay_recent_measurement_attempt_count': '1',
  'relay_recent_measurements_excluded_error_count': '1',
  'relay_recent_priority_list_count': '1',
  'success': '3',
}

EXPECTED_NEW_HEADER_CONTENT = """
1410723598
version=1.1.0
new_header=neat stuff
=====
""".strip()

WRONG_VERSION_POSITION = b"""
1410723598
file_created=2019-01-14T05:35:06
version=1.1.0
=====
""".strip()

RIGHT_VERSION_POSITION = b"""
1410723598
version=1.1.0
file_created=2019-01-14T05:35:06
=====
""".strip()


class TestBandwidthFile(unittest.TestCase):
  def test_from_str(self):
    sig = BandwidthFile.create()
    self.assertEqual(sig, BandwidthFile.from_str(str(sig)))

  def test_format_v1_0(self):
    """
    Parse version 1.0 formatted files.
    """

    desc = list(stem.descriptor.parse_file(get_resource('bandwidth_file_v1.0'), 'bandwidth-file 1.0'))[0]

    self.assertEqual(datetime.datetime(2019, 1, 14, 17, 41, 29), desc.timestamp)
    self.assertEqual('1.0.0', desc.version)

    self.assertEqual(None, desc.software)
    self.assertEqual(None, desc.software_version)

    self.assertEqual(None, desc.earliest_bandwidth)
    self.assertEqual(None, desc.latest_bandwidth)
    self.assertEqual(None, desc.created_at)
    self.assertEqual(None, desc.generated_at)

    self.assertEqual(None, desc.consensus_size)
    self.assertEqual(None, desc.eligible_count)
    self.assertEqual(None, desc.eligible_percent)
    self.assertEqual(None, desc.min_count)
    self.assertEqual(None, desc.min_percent)

    self.assertEqual(None, desc.scanner_country)
    self.assertEqual(None, desc.destinations_countries)
    self.assertEqual(None, desc.time_to_report_half_network)

    stats = desc.recent_stats
    self.assertEqual(None, stats.consensus_count)
    self.assertEqual(None, stats.prioritized_relays)
    self.assertEqual(None, stats.prioritized_relay_lists)
    self.assertEqual(None, stats.measurement_attempts)
    self.assertEqual(None, stats.measurement_failures)

    relay_failures = stats.relay_failures
    self.assertEqual(None, relay_failures.no_measurement)
    self.assertEqual(None, relay_failures.insuffient_period)
    self.assertEqual(None, relay_failures.insufficient_measurements)
    self.assertEqual(None, relay_failures.stale)

    self.assertEqual(94, len(desc.measurements))
    self.assertEqual(EXPECTED_MEASUREMENT_1, desc.measurements['D8B9CAA5B818DEFE80857F83FDABBB6429DCFCA0'])

  def test_format_v1_2(self):
    """
    Parse version 1.2 formatted files.
    """

    desc = list(stem.descriptor.parse_file(get_resource('bandwidth_file_v1.2'), 'bandwidth-file 1.2'))[0]

    self.assertEqual(datetime.datetime(2019, 1, 14, 5, 34, 59), desc.timestamp)
    self.assertEqual('1.2.0', desc.version)

    self.assertEqual('sbws', desc.software)
    self.assertEqual('1.0.2', desc.software_version)

    self.assertEqual(datetime.datetime(2019, 1, 4, 5, 35, 29), desc.earliest_bandwidth)
    self.assertEqual(datetime.datetime(2019, 1, 14, 5, 34, 59), desc.latest_bandwidth)
    self.assertEqual(datetime.datetime(2019, 1, 14, 5, 35, 6), desc.created_at)
    self.assertEqual(datetime.datetime(2019, 1, 3, 22, 45, 8), desc.generated_at)

    self.assertEqual(6514, desc.consensus_size)
    self.assertEqual(6256, desc.eligible_count)
    self.assertEqual(96, desc.eligible_percent)
    self.assertEqual(3908, desc.min_count)
    self.assertEqual(60, desc.min_percent)

    self.assertEqual(None, desc.scanner_country)
    self.assertEqual(None, desc.destinations_countries)
    self.assertEqual(None, desc.time_to_report_half_network)

    stats = desc.recent_stats
    self.assertEqual(None, stats.consensus_count)
    self.assertEqual(None, stats.prioritized_relays)
    self.assertEqual(None, stats.prioritized_relay_lists)
    self.assertEqual(None, stats.measurement_attempts)
    self.assertEqual(None, stats.measurement_failures)

    relay_failures = stats.relay_failures
    self.assertEqual(None, relay_failures.no_measurement)
    self.assertEqual(None, relay_failures.insuffient_period)
    self.assertEqual(None, relay_failures.insufficient_measurements)
    self.assertEqual(None, relay_failures.stale)

    self.assertEqual(81, len(desc.measurements))
    self.assertEqual(EXPECTED_MEASUREMENT_2, desc.measurements['9C7E1AFDACC53228F6FB57B3A08C7D36240B8F6F'])

  def test_format_v1_4(self):
    """
    Parse version 1.4 formatted files.
    """

    desc = list(stem.descriptor.parse_file(get_resource('bandwidth_file_v1.4'), 'bandwidth-file 1.4'))[0]

    self.assertEqual(datetime.datetime(2019, 4, 21, 21, 34, 57), desc.timestamp)
    self.assertEqual('1.4.0', desc.version)

    self.assertEqual('sbws', desc.software)
    self.assertEqual('1.1.0', desc.software_version)

    self.assertEqual(datetime.datetime(2019, 4, 16, 21, 35, 7), desc.earliest_bandwidth)
    self.assertEqual(datetime.datetime(2019, 4, 21, 21, 34, 57), desc.latest_bandwidth)
    self.assertEqual(datetime.datetime(2019, 4, 21, 21, 35, 4), desc.created_at)
    self.assertEqual(datetime.datetime(2019, 4, 20, 11, 40, 1), desc.generated_at)

    self.assertEqual(6684, desc.consensus_size)
    self.assertEqual(6459, desc.eligible_count)
    self.assertEqual(97, desc.eligible_percent)
    self.assertEqual(4010, desc.min_count)
    self.assertEqual(60, desc.min_percent)

    self.assertEqual('US', desc.scanner_country)
    self.assertEqual(['ZZ'], desc.destinations_countries)
    self.assertEqual(223519, desc.time_to_report_half_network)

    stats = desc.recent_stats
    self.assertEqual(34, stats.consensus_count)
    self.assertEqual(86417, stats.prioritized_relays)
    self.assertEqual(260, stats.prioritized_relay_lists)
    self.assertEqual(86417, stats.measurement_attempts)
    self.assertEqual(57023, stats.measurement_failures)

    relay_failures = stats.relay_failures
    self.assertEqual(788, relay_failures.no_measurement)
    self.assertEqual(182, relay_failures.insuffient_period)
    self.assertEqual(663, relay_failures.insufficient_measurements)
    self.assertEqual(0, relay_failures.stale)

    self.assertEqual(58, len(desc.measurements))
    self.assertEqual(EXPECTED_MEASUREMENT_3, desc.measurements['8F0F49F2341C7F706D5B475815DBD3E5761334B3'])

  @patch('time.time', Mock(return_value = 1410723598.276578))
  def test_minimal_bandwidth_file(self):
    """
    Basic sanity check that we can parse a bandwidth file with minimal
    attributes.
    """

    desc = BandwidthFile.create()

    self.assertEqual('1410723598', str(desc))

    self.assertEqual(datetime.datetime(2014, 9, 14, 19, 39, 58), desc.timestamp)
    self.assertEqual('1.0.0', desc.version)

    self.assertEqual(None, desc.software)
    self.assertEqual(None, desc.software_version)

    self.assertEqual(None, desc.earliest_bandwidth)
    self.assertEqual(None, desc.latest_bandwidth)
    self.assertEqual(None, desc.created_at)
    self.assertEqual(None, desc.generated_at)

    self.assertEqual(None, desc.consensus_size)
    self.assertEqual(None, desc.eligible_count)
    self.assertEqual(None, desc.eligible_percent)
    self.assertEqual(None, desc.min_count)
    self.assertEqual(None, desc.min_percent)

    self.assertEqual({}, desc.header)

  def test_content_example(self):
    """
    Exercise the example in our content method's pydoc.
    """

    content = BandwidthFile.content(OrderedDict([
      ('timestamp', '12345'),
      ('version', '1.2.0'),
      ('content', []),
    ]))

    self.assertEqual(b'12345\nversion=1.2.0\n=====', content)

  @patch('time.time', Mock(return_value = 1410723598.276578))
  def test_new_header_attribute(self):
    """
    Include an unrecognized header field.
    """

    desc = BandwidthFile.create(OrderedDict([('version', '1.1.0'), ('new_header', 'neat stuff')]))
    self.assertEqual(EXPECTED_NEW_HEADER_CONTENT, str(desc))
    self.assertEqual('1.1.0', desc.version)
    self.assertEqual({'version': '1.1.0', 'new_header': 'neat stuff'}, desc.header)

  def test_header_for_v1(self):
    """
    Document version 1.0 predates headers, and as such should be prohibited.
    """

    self.assertRaisesWith(ValueError, 'Headers require BandwidthFile version 1.1 or later', BandwidthFile.create, {'new_header': 'neat stuff'})

  def test_version_position(self):
    """
    Our 'version' header must be in the second position if validated, but
    otherwise doesn't matter. (:trac:`29539`)
    """

    desc = BandwidthFile.from_str(WRONG_VERSION_POSITION)
    self.assertEqual('1.1.0', desc.version)

    self.assertRaisesWith(ValueError, "The 'version' header must be in the second position", BandwidthFile.from_str, WRONG_VERSION_POSITION, validate = True)

    content = BandwidthFile.content(OrderedDict([
      ('timestamp', '1410723598'),
      ('file_created', '2019-01-14T05:35:06'),
      ('version', '1.1.0'),
      ('content', []),
    ]))

    self.assertEqual(RIGHT_VERSION_POSITION, content)

  def test_header_alternate_div(self):
    """
    To support backward compatability four character dividers are allowed.
    """

    with open(get_resource('bandwidth_file_v1.2')) as desc_file:
      desc = BandwidthFile.from_str(desc_file.read().replace('=====', '===='))

    self.assertEqual(datetime.datetime(2019, 1, 14, 5, 34, 59), desc.timestamp)
    self.assertEqual('1.2.0', desc.version)
    self.assertEqual(81, len(desc.measurements))

  def test_invalid_timestamp(self):
    """
    Invalid timestamp values.
    """

    test_values = (
      b'',
      b'boo',
      b'123.4',
      b'-123',
    )

    for value in test_values:
      expected_exc = "First line should be a unix timestamp, but was '%s'" % value
      self.assertRaisesWith(ValueError, expected_exc, BandwidthFile.create, {'timestamp': value})



"""
Unit tests for the stem.connection.authenticate function.

Under the covers the authentiate function really just translates a
PROTOCOLINFO response into authenticate_* calls, then does prioritization
on the exceptions if they all fail.

This monkey patches the various functions authenticate relies on to exercise
various error conditions, and make sure that the right exception is raised.
"""

import unittest

import stem.connection
import test

from stem.response import ControlMessage
from stem.util import log

try:
  # added in python 3.3
  from unittest.mock import Mock, patch
except ImportError:
  from mock import Mock, patch


class TestAuthenticate(unittest.TestCase):
  @patch('stem.connection.get_protocolinfo')
  @patch('stem.connection.authenticate_none', Mock())
  def test_with_get_protocolinfo(self, get_protocolinfo_mock):
    """
    Tests the authenticate() function when it needs to make a get_protocolinfo.
    """

    # tests where get_protocolinfo succeeds

    protocolinfo_message = ControlMessage.from_str('250-PROTOCOLINFO 1\r\n250 OK\r\n', 'PROTOCOLINFO')
    protocolinfo_message.auth_methods = (stem.connection.AuthMethod.NONE, )
    get_protocolinfo_mock.return_value = protocolinfo_message

    stem.connection.authenticate(None)

    # tests where get_protocolinfo raises an exception

    get_protocolinfo_mock.side_effect = stem.ProtocolError
    self.assertRaises(stem.connection.IncorrectSocketType, stem.connection.authenticate, None)

    get_protocolinfo_mock.side_effect = stem.SocketError
    self.assertRaises(stem.connection.AuthenticationFailure, stem.connection.authenticate, None)

  @patch('stem.connection.authenticate_none')
  @patch('stem.connection.authenticate_password')
  @patch('stem.connection.authenticate_cookie')
  @patch('stem.connection.authenticate_safecookie')
  def test_all_use_cases(self, authenticate_safecookie_mock, authenticate_cookie_mock, authenticate_password_mock, authenticate_none_mock):
    """
    Does basic validation that all valid use cases for the PROTOCOLINFO input
    and dependent functions result in either success or a AuthenticationFailed
    subclass being raised.
    """

    # mute the logger for this test since otherwise the output is overwhelming

    stem_logger = log.get_logger()
    stem_logger.setLevel(log.logging_level(None))

    # exceptions that the authentication functions are documented to raise

    all_auth_none_exc = (
      None,
      stem.connection.OpenAuthRejected(None),
      stem.ControllerError(None))

    all_auth_password_exc = (
      None,
      stem.connection.PasswordAuthRejected(None),
      stem.connection.IncorrectPassword(None),
      stem.ControllerError(None))

    all_auth_cookie_exc = (
      None,
      stem.connection.CookieAuthRejected(None, False, None),
      stem.connection.IncorrectCookieValue(None, False, None),
      stem.connection.IncorrectCookieSize(None, False, None),
      stem.connection.UnreadableCookieFile(None, False, None),
      stem.connection.AuthChallengeFailed(None, None),
      stem.ControllerError(None))

    auth_method_combinations = test.get_all_combinations([
      stem.connection.AuthMethod.NONE,
      stem.connection.AuthMethod.PASSWORD,
      stem.connection.AuthMethod.COOKIE,
      stem.connection.AuthMethod.SAFECOOKIE,
      stem.connection.AuthMethod.UNKNOWN,
    ], include_empty = True)

    protocolinfo = ControlMessage.from_str('250-PROTOCOLINFO 1\r\n250 OK\r\n', 'PROTOCOLINFO')
    protocolinfo.cookie_path = '/tmp/blah'

    for auth_methods in auth_method_combinations:
      for auth_none_exc in all_auth_none_exc:
        for auth_password_exc in all_auth_password_exc:
          for auth_cookie_exc in all_auth_cookie_exc:
            # Skip iteration if it's to test exceptions for authentication
            # we're not using.

            if auth_none_exc and stem.connection.AuthMethod.NONE not in auth_methods:
              continue
            elif auth_password_exc and stem.connection.AuthMethod.PASSWORD not in auth_methods:
              continue
            elif auth_cookie_exc and stem.connection.AuthMethod.COOKIE not in auth_methods and stem.connection.AuthMethod.SAFECOOKIE not in auth_methods:
              continue

            # Determine if the authenticate() call will succeed and mock each
            # of the authenticate_* function to raise its given exception.
            #
            # This implementation is slightly inaccurate in a couple regards...
            # a. it raises safecookie exceptions from authenticate_cookie()
            # b. exceptions raised by authenticate_cookie() and
            #    authenticate_safecookie() are always the same
            #
            # However, adding another loop for safe_cookie exceptions means
            # multiplying our runtime many fold. This exercises everything that
            # matters so the above inaccuracies seem fine.

            expect_success = False
            protocolinfo.auth_methods = auth_methods

            for auth_method in auth_methods:
              if auth_method == stem.connection.AuthMethod.NONE:
                auth_mock, raised_exc = authenticate_none_mock, auth_none_exc
              elif auth_method == stem.connection.AuthMethod.PASSWORD:
                auth_mock, raised_exc = authenticate_password_mock, auth_password_exc
              elif auth_method == stem.connection.AuthMethod.COOKIE:
                auth_mock, raised_exc = authenticate_cookie_mock, auth_cookie_exc
              elif auth_method == stem.connection.AuthMethod.SAFECOOKIE:
                auth_mock, raised_exc = authenticate_safecookie_mock, auth_cookie_exc

              if raised_exc:
                auth_mock.side_effect = raised_exc
              else:
                auth_mock.side_effect = None
                expect_success = True

            if expect_success:
              stem.connection.authenticate(None, 'blah', None, protocolinfo)
            else:
              self.assertRaises(stem.connection.AuthenticationFailure, stem.connection.authenticate, None, 'blah', None, protocolinfo)

    # revert logging back to normal
    stem_logger.setLevel(log.logging_level(log.TRACE))



"""
Unit tests for the stem.response.SingleLineResponse class.
"""

import unittest

import stem

from stem.response import ControlMessage


class TestSingleLineResponse(unittest.TestCase):
  def test_single_line_response(self):
    message = ControlMessage.from_str('552 NOTOK\r\n', 'SINGLELINE')
    self.assertEqual(False, message.is_ok())

    message = ControlMessage.from_str('250 KK\r\n', 'SINGLELINE')
    self.assertEqual(True, message.is_ok())

    message = ControlMessage.from_str('250 OK\r\n', 'SINGLELINE')
    self.assertEqual(True, message.is_ok(True))

    message = ControlMessage.from_str('250 HMM\r\n', 'SINGLELINE')
    self.assertEqual(False, message.is_ok(True))

  def test_multi_line_response(self):
    self.assertRaises(stem.ProtocolError, ControlMessage.from_str, '250-MULTI\r\n250 LINE\r\n', 'SINGLELINE')



"""
Unit tests for stem.client.Address.
"""

import collections
import unittest

from stem.client.datatype import AddrType, Address

ExpectedAddress = collections.namedtuple('ExpectedAddress', ['type', 'type_int', 'value', 'value_bin'])


class TestAddress(unittest.TestCase):
  def test_enum(self):
    self.assertEqual(('IPv4', 4), AddrType.get(AddrType.IPv4))
    self.assertEqual(('IPv4', 4), AddrType.get(4))

    self.assertEqual(('UNKNOWN', 25), AddrType.get(25))
    self.assertRaisesWith(ValueError, "Invalid enumeration 'boom', options are HOSTNAME, IPv4, IPv6, ERROR_TRANSIENT, ERROR_PERMANENT, UNKNOWN", AddrType.get, 'boom')

  def test_constructor(self):
    test_data = (
      ((4, b'\x7f\x00\x00\x01'), ExpectedAddress(AddrType.IPv4, 4, '127.0.0.1', b'\x7f\x00\x00\x01')),
      ((4, b'aq\x0f\x02'), ExpectedAddress(AddrType.IPv4, 4, '97.113.15.2', b'aq\x0f\x02')),
      ((6, b' \x01\r\xb8\x00\x00\x00\x00\x00\x00\xff\x00\x00B\x83)'), ExpectedAddress(AddrType.IPv6, 6, '2001:0db8:0000:0000:0000:ff00:0042:8329', b' \x01\r\xb8\x00\x00\x00\x00\x00\x00\xff\x00\x00B\x83)')),
      ((AddrType.IPv4, '127.0.0.1'), ExpectedAddress(AddrType.IPv4, 4, '127.0.0.1', b'\x7f\x00\x00\x01')),
      ((AddrType.IPv4, '97.113.15.2'), ExpectedAddress(AddrType.IPv4, 4, '97.113.15.2', b'aq\x0f\x02')),
      ((AddrType.IPv6, '2001:0db8:0000:0000:0000:ff00:0042:8329'), ExpectedAddress(AddrType.IPv6, 6, '2001:0db8:0000:0000:0000:ff00:0042:8329', b' \x01\r\xb8\x00\x00\x00\x00\x00\x00\xff\x00\x00B\x83)')),
      ((AddrType.IPv6, '2001:0DB8:AC10:FE01::'), ExpectedAddress(AddrType.IPv6, 6, '2001:0db8:ac10:fe01:0000:0000:0000:0000', b' \x01\r\xb8\xac\x10\xfe\x01\x00\x00\x00\x00\x00\x00\x00\x00')),  # collaped and different case
    )

    for (addr_type, addr_value), expected in test_data:
      addr = Address(addr_value, addr_type)
      self.assertEqual(expected.type, addr.type)
      self.assertEqual(expected.type_int, addr.type_int)
      self.assertEqual(expected.value, addr.value)
      self.assertEqual(expected.value_bin, addr.value_bin)

    # when an IPv4 or IPv6 address the type is optional

    self.assertEqual(AddrType.IPv4, Address('127.0.0.1').type)
    self.assertEqual(AddrType.IPv6, Address('2001:0DB8:AC10:FE01::').type)

    self.assertRaisesWith(ValueError, "Packed IPv4 addresses should be four bytes, but was: '\\x7f\\x00'", Address, '\x7f\x00', 4)
    self.assertRaisesWith(ValueError, "Packed IPv6 addresses should be sixteen bytes, but was: '\\x7f\\x00'", Address, '\x7f\x00', 6)
    self.assertRaisesWith(ValueError, "'nope' isn't an IPv4 or IPv6 address", Address, 'nope')

  def test_unknown_type(self):
    addr = Address('hello', 12)
    self.assertEqual(AddrType.UNKNOWN, addr.type)
    self.assertEqual(12, addr.type_int)
    self.assertEqual(None, addr.value)
    self.assertEqual('hello', addr.value_bin)

  def test_packing(self):
    test_data = {
      b'\x04\x04\x7f\x00\x00\x01': Address('127.0.0.1'),
      b'\x06\x10 \x01\r\xb8\x00\x00\x00\x00\x00\x00\xff\x00\x00B\x83)': Address('2001:0db8:0000:0000:0000:ff00:0042:8329'),
    }

    for cell_bytes, address in test_data.items():
      self.assertEqual(cell_bytes, address.pack())
      self.assertEqual(address, Address.unpack(cell_bytes))

    addr, content = Address.pop(b'\x04\x04\x7f\x00\x00\x01\x01\x04\x04aq\x0f\x02\x00\x00\x00\x00')
    self.assertEqual(b'\x01\x04\x04aq\x0f\x02\x00\x00\x00\x00', content)

    self.assertEqual(AddrType.IPv4, addr.type)
    self.assertEqual(4, addr.type_int)
    self.assertEqual('127.0.0.1', addr.value)
    self.assertEqual(b'\x7f\x00\x00\x01', addr.value_bin)



# Copyright 2015-2019, Damian Johnson and The Tor Project
# See LICENSE for licensing information

"""
Commandline argument parsing for our test runner.
"""

import collections
import getopt

import stem.util.conf
import stem.util.log
import test

LOG_TYPE_ERROR = """\
'%s' isn't a logging runlevel, use one of the following instead:
  TRACE, DEBUG, INFO, NOTICE, WARN, ERROR
"""

CONFIG = stem.util.conf.config_dict('test', {
  'msg.help': '',
  'target.description': {},
  'target.torrc': {},
})

DEFAULT_ARGS = {
  'run_unit': False,
  'run_integ': False,
  'specific_test': [],
  'logging_runlevel': None,
  'logging_path': None,
  'tor_path': 'tor',
  'run_targets': [test.Target.RUN_OPEN],
  'attribute_targets': [],
  'quiet': False,
  'verbose': False,
  'print_help': False,
}

OPT = 'auit:l:qvh'
OPT_EXPANDED = ['all', 'unit', 'integ', 'targets=', 'test=', 'log=', 'log-file=', 'tor=', 'quiet', 'verbose', 'help']


def parse(argv):
  """
  Parses our arguments, providing a named tuple with their values.

  :param list argv: input arguments to be parsed

  :returns: a **named tuple** with our parsed arguments

  :raises: **ValueError** if we got an invalid argument
  """

  args = dict(DEFAULT_ARGS)

  try:
    recognized_args, unrecognized_args = getopt.getopt(argv, OPT, OPT_EXPANDED)

    if unrecognized_args:
      error_msg = "aren't recognized arguments" if len(unrecognized_args) > 1 else "isn't a recognized argument"
      raise getopt.GetoptError("'%s' %s" % ("', '".join(unrecognized_args), error_msg))
  except Exception as exc:
    raise ValueError('%s (for usage provide --help)' % exc)

  for opt, arg in recognized_args:
    if opt in ('-a', '--all'):
      args['run_unit'] = True
      args['run_integ'] = True
    elif opt in ('-u', '--unit'):
      args['run_unit'] = True
    elif opt in ('-i', '--integ'):
      args['run_integ'] = True
    elif opt in ('-t', '--targets'):
      run_targets, attribute_targets = [], []

      integ_targets = arg.split(',')
      all_run_targets = [t for t in test.Target if CONFIG['target.torrc'].get(t) is not None]

      # validates the targets and split them into run and attribute targets

      if not integ_targets:
        raise ValueError('No targets provided')

      for target in integ_targets:
        if target not in test.Target:
          raise ValueError('Invalid integration target: %s' % target)
        elif target in all_run_targets:
          run_targets.append(target)
        else:
          attribute_targets.append(target)

      # check if we were told to use all run targets

      if test.Target.RUN_ALL in attribute_targets:
        attribute_targets.remove(test.Target.RUN_ALL)
        run_targets = all_run_targets

      # if no RUN_* targets are provided then keep the default (otherwise we
      # won't have any tests to run)

      if run_targets:
        args['run_targets'] = run_targets

      args['attribute_targets'] = attribute_targets
    elif opt == '--test':
      args['specific_test'].append(crop_module_name(arg))
    elif opt in ('-l', '--log'):
      arg = arg.upper()

      if arg not in stem.util.log.LOG_VALUES:
        raise ValueError(LOG_TYPE_ERROR % arg)

      args['logging_runlevel'] = arg
    elif opt == '--log-file':
      args['logging_path'] = arg
    elif opt in ('--tor'):
      args['tor_path'] = arg
    elif opt in ('-q', '--quiet'):
      args['quiet'] = True
    elif opt in ('-v', '--verbose'):
      args['verbose'] = True
    elif opt in ('-h', '--help'):
      args['print_help'] = True

  # translates our args dict into a named tuple

  Args = collections.namedtuple('Args', args.keys())
  return Args(**args)


def get_help():
  """
  Provides usage information, as provided by the '--help' argument. This
  includes a listing of the valid integration targets.

  :returns: **str** with our usage information
  """

  help_msg = CONFIG['msg.help']

  # gets the longest target length so we can show the entries in columns
  target_name_length = max(map(len, test.Target))
  description_format = '\n    %%-%is - %%s' % target_name_length

  for target in test.Target:
    help_msg += description_format % (target, CONFIG['target.description'].get(target, ''))

  help_msg += '\n'

  return help_msg


def crop_module_name(name):
  """
  Test modules have a 'test.unit.' or 'test.integ.' prefix which can
  be omitted from our '--test' argument. Cropping this so we can do
  normalized comparisons.

  :param str name: module name to crop

  :returns: **str** with the cropped module name
  """

  if name.startswith('test.unit.'):
    return name[10:]
  elif name.startswith('test.integ.'):
    return name[11:]
  else:
    return name



"""
Unit tests for stem.descriptor.collector.
"""

import datetime
import io
import unittest

import stem.prereq

from stem.descriptor import Compression, DocumentHandler
from stem.descriptor.collector import CollecTor, File
from test.unit.descriptor import get_resource
from test.unit.descriptor.data.collector.index import EXAMPLE_INDEX

try:
  # added in python 3.3
  from unittest.mock import Mock, patch
except ImportError:
  from mock import Mock, patch

URL_OPEN = 'urllib.request.urlopen' if stem.prereq.is_python_3() else 'urllib2.urlopen'


with open(get_resource('collector/index.json'), 'rb') as index_file:
  EXAMPLE_INDEX_JSON = index_file.read()


class TestCollector(unittest.TestCase):
  # tests for the File class

  def test_file_guess_compression(self):
    test_values = {
      'archive/relay-descriptors/microdescs/microdescs-2014-01.tar.xz': Compression.LZMA,
      'archive/webstats/webstats-2015-03.tar': Compression.PLAINTEXT,
      'recent/relay-descriptors/extra-infos/2019-07-03-02-05-00-extra-infos': Compression.PLAINTEXT,
    }

    for path, expected in test_values.items():
      self.assertEqual(expected, File._guess_compression(path))

  def test_file_guess_time_range(self):
    test_values = {
      'archive/relay-descriptors/microdescs/microdescs-2014-01.tar.xz':
        (datetime.datetime(2014, 1, 1), datetime.datetime(2014, 2, 1)),
      'recent/relay-descriptors/extra-infos/2019-07-03-02-05-00-extra-infos':
        (datetime.datetime(2019, 7, 3, 2, 5, 0), datetime.datetime(2019, 7, 3, 3, 5, 0)),
      'archive/relay-descriptors/certs.tar.xz':
        (None, None),
      'archive/relay-descriptors/microdescs/microdescs-2014-12.tar.xz':
        (datetime.datetime(2014, 12, 1), datetime.datetime(2015, 1, 1)),
      'recent/relay-descriptors/extra-infos/2019-07-03-23-05-00-extra-infos':
        (datetime.datetime(2019, 7, 3, 23, 5, 0), datetime.datetime(2019, 7, 4, 0, 5, 0))
    }

    for path, (expected_start, expected_end) in test_values.items():
      f = File(path, [], 7515396, 'BVVDEkegsLzkAn30dYikr4yTT79+XScfc0VUVEb83tM=', None, None, '2019-07-29 18:45')
      self.assertEqual(expected_start, f.start)
      self.assertEqual(expected_end, f.end)

  # tests for the CollecTor class

  @patch(URL_OPEN)
  def test_index_plaintext(self, urlopen_mock):
    urlopen_mock.return_value = io.BytesIO(EXAMPLE_INDEX_JSON)

    collector = CollecTor()
    self.assertEqual(EXAMPLE_INDEX, collector.index(Compression.PLAINTEXT))
    urlopen_mock.assert_called_with('https://collector.torproject.org/index/index.json', timeout = None)

  @patch(URL_OPEN)
  def test_index_gzip(self, urlopen_mock):
    if not Compression.GZIP.available:
      self.skipTest('(gzip compression unavailable)')
      return

    import zlib
    urlopen_mock.return_value = io.BytesIO(zlib.compress(EXAMPLE_INDEX_JSON))

    collector = CollecTor()
    self.assertEqual(EXAMPLE_INDEX, collector.index(Compression.GZIP))
    urlopen_mock.assert_called_with('https://collector.torproject.org/index/index.json.gz', timeout = None)

  @patch(URL_OPEN)
  def test_index_bz2(self, urlopen_mock):
    if not Compression.BZ2.available:
      self.skipTest('(bz2 compression unavailable)')
      return

    import bz2
    urlopen_mock.return_value = io.BytesIO(bz2.compress(EXAMPLE_INDEX_JSON))

    collector = CollecTor()
    self.assertEqual(EXAMPLE_INDEX, collector.index(Compression.BZ2))
    urlopen_mock.assert_called_with('https://collector.torproject.org/index/index.json.bz2', timeout = None)

  @patch(URL_OPEN)
  def test_index_lzma(self, urlopen_mock):
    if not Compression.LZMA.available:
      self.skipTest('(lzma compression unavailable)')
      return

    import lzma
    urlopen_mock.return_value = io.BytesIO(lzma.compress(EXAMPLE_INDEX_JSON))

    collector = CollecTor()
    self.assertEqual(EXAMPLE_INDEX, collector.index(Compression.LZMA))
    urlopen_mock.assert_called_with('https://collector.torproject.org/index/index.json.xz', timeout = None)

  @patch(URL_OPEN)
  def test_index_retries(self, urlopen_mock):
    urlopen_mock.side_effect = IOError('boom')

    collector = CollecTor(retries = 0)
    self.assertRaisesRegexp(IOError, 'boom', collector.index)
    self.assertEqual(1, urlopen_mock.call_count)

    urlopen_mock.reset_mock()

    collector = CollecTor(retries = 4)
    self.assertRaisesRegexp(IOError, 'boom', collector.index)
    self.assertEqual(5, urlopen_mock.call_count)

  @patch(URL_OPEN, Mock(return_value = io.BytesIO(b'not json')))
  def test_index_malformed_json(self):
    collector = CollecTor()

    if stem.prereq.is_python_3():
      self.assertRaisesRegexp(ValueError, 'Expecting value: line 1 column 1', collector.index, Compression.PLAINTEXT)
    else:
      self.assertRaisesRegexp(ValueError, 'No JSON object could be decoded', collector.index, Compression.PLAINTEXT)

  def test_index_malformed_compression(self):
    for compression in (Compression.GZIP, Compression.BZ2, Compression.LZMA):
      if not compression.available:
        continue

      with patch(URL_OPEN, Mock(return_value = io.BytesIO(b'not compressed'))):
        collector = CollecTor()
        self.assertRaisesRegexp(IOError, 'Failed to decompress as %s' % compression, collector.index, compression)

  @patch('stem.descriptor.collector.CollecTor.index', Mock(return_value = EXAMPLE_INDEX))
  def test_files(self):
    collector = CollecTor()
    files = collector.files()
    self.assertEqual(96, len(files))

    extrainfo_file = list(filter(lambda x: x.path.endswith('extra-infos-2007-09.tar.xz'), files))[0]
    self.assertEqual('archive/relay-descriptors/extra-infos/extra-infos-2007-09.tar.xz', extrainfo_file.path)
    self.assertEqual(Compression.LZMA, extrainfo_file.compression)
    self.assertEqual(6459884, extrainfo_file.size)
    self.assertEqual(datetime.datetime(2016, 6, 23, 9, 54), extrainfo_file.last_modified)

  @patch('stem.descriptor.collector.CollecTor.index', Mock(return_value = EXAMPLE_INDEX))
  def test_files_by_descriptor_type(self):
    collector = CollecTor()

    self.assertEqual([
      'archive/relay-descriptors/server-descriptors/server-descriptors-2005-12.tar.xz',
      'archive/relay-descriptors/server-descriptors/server-descriptors-2006-02.tar.xz',
      'archive/relay-descriptors/server-descriptors/server-descriptors-2006-03.tar.xz',
      'recent/relay-descriptors/server-descriptors/2019-11-28-01-05-00-server-descriptors',
      'recent/relay-descriptors/server-descriptors/2019-11-28-00-05-00-server-descriptors',
      'recent/relay-descriptors/server-descriptors/2019-11-27-23-05-00-server-descriptors',
    ], [f.path for f in collector.files(descriptor_type = 'server-descriptor')])

  @patch('stem.descriptor.collector.CollecTor.index', Mock(return_value = EXAMPLE_INDEX))
  def test_file_by_date(self):
    collector = CollecTor()

    self.assertEqual([
      'recent/relay-descriptors/server-descriptors/2019-11-28-01-05-00-server-descriptors',
      'recent/relay-descriptors/server-descriptors/2019-11-28-00-05-00-server-descriptors',
      'recent/relay-descriptors/server-descriptors/2019-11-27-23-05-00-server-descriptors',
    ], [f.path for f in collector.files(descriptor_type = 'server-descriptor', start = datetime.datetime(2007, 1, 1))])

    self.assertEqual([
      'archive/relay-descriptors/server-descriptors/server-descriptors-2005-12.tar.xz',
      'archive/relay-descriptors/server-descriptors/server-descriptors-2006-02.tar.xz',
      'archive/relay-descriptors/server-descriptors/server-descriptors-2006-03.tar.xz',
    ], [f.path for f in collector.files(descriptor_type = 'server-descriptor', end = datetime.datetime(2007, 1, 1))])

    self.assertEqual([
      'archive/relay-descriptors/server-descriptors/server-descriptors-2006-02.tar.xz',
      'archive/relay-descriptors/server-descriptors/server-descriptors-2006-03.tar.xz',
    ], [f.path for f in collector.files(descriptor_type = 'server-descriptor', start = datetime.datetime(2006, 2, 10), end = datetime.datetime(2007, 1, 1))])

  @patch('stem.util.connection.download')
  @patch('stem.descriptor.collector.CollecTor.files')
  def test_reading_server_descriptors(self, files_mock, download_mock):
    with open(get_resource('collector/server-descriptors-2005-12-cropped.tar'), 'rb') as archive:
      download_mock.return_value = archive.read()

    files_mock.return_value = [stem.descriptor.collector.File(
      'archive/relay-descriptors/server-descriptors/server-descriptors-2005-12.tar',
      ['server-descriptor 1.0'],
      1348620,
      'v3ANi2FD4xAhmyzigQq9gvlLwpXH8I6fGoiYlWLjOy8=',
      '2005-12-15 01:42',
      '2005-12-17 11:06',
      '2016-06-24 08:12',
    )]

    descriptors = list(stem.descriptor.collector.get_server_descriptors())
    self.assertEqual(5, len(descriptors))

    f = descriptors[0]
    self.assertEqual('RelayDescriptor', type(f).__name__)
    self.assertEqual('3E2F63E2356F52318B536A12B6445373808A5D6C', f.fingerprint)

  @patch('stem.util.connection.download')
  @patch('stem.descriptor.collector.CollecTor.files')
  def test_reading_bridge_server_descriptors(self, files_mock, download_mock):
    with open(get_resource('collector/bridge-server-descriptors-2019-02-cropped.tar'), 'rb') as archive:
      download_mock.return_value = archive.read()

    files_mock.return_value = [stem.descriptor.collector.File(
      'archive/bridge-descriptors/server-descriptors/bridge-server-descriptors-2008-05.tar',
      ['bridge-server-descriptor 1.2'],
      205348,
      'NRb2dzS2OhFKYjfr5WoleOFokqC4C+qf0Nu4iegnFLo=',
      '2008-05-14 18:22',
      '2008-05-31 23:09',
      '2016-09-09 14:13',
    )]

    descriptors = list(stem.descriptor.collector.get_server_descriptors(bridge = True))
    self.assertEqual(4, len(descriptors))

    f = descriptors[0]
    self.assertEqual('BridgeDescriptor', type(f).__name__)
    self.assertEqual('E90D1DE12B930DEC3F3E1127AAA25E47430CD3F4', f.fingerprint)

  @patch('stem.util.connection.download')
  @patch('stem.descriptor.collector.CollecTor.files')
  def test_reading_extrainfo_descriptors(self, files_mock, download_mock):
    with open(get_resource('collector/extra-infos-2019-04-cropped.tar'), 'rb') as archive:
      download_mock.return_value = archive.read()

    files_mock.return_value = [stem.descriptor.collector.File(
      'archive/relay-descriptors/extra-infos/extra-infos-2007-08.tar',
      ['extra-info 1.0'],
      3016916,
      'UcAIrzYjFU52mRHXNle/fbI21lvfsVkeC0NpBZ/Pt/w=',
      '2007-08-14 17:35',
      '2007-08-31 23:53',
      '2016-06-23 09:53',
    )]

    descriptors = list(stem.descriptor.collector.get_extrainfo_descriptors())
    self.assertEqual(7, len(descriptors))

    f = descriptors[0]
    self.assertEqual('RelayExtraInfoDescriptor', type(f).__name__)
    self.assertEqual('170EF19C0FA0491DFCEA6E1FB0941670B80506E1', f.fingerprint)

  @patch('stem.util.connection.download')
  @patch('stem.descriptor.collector.CollecTor.files')
  def test_reading_bridge_extrainfo_descriptors(self, files_mock, download_mock):
    with open(get_resource('collector/bridge-extra-infos-2019-03-cropped.tar'), 'rb') as archive:
      download_mock.return_value = archive.read()

    files_mock.return_value = [stem.descriptor.collector.File(
      'archive/bridge-descriptors/extra-infos/bridge-extra-infos-2008-05.tar',
      ['bridge-extra-info 1.3'],
      377644,
      'aDD2q7uNGOM+WuH67+nTd7rvFN4P580xPAmXYtqxr2I=',
      '2008-05-13 15:21',
      '2008-05-31 23:09',
      '2016-09-04 09:21',
    )]

    descriptors = list(stem.descriptor.collector.get_extrainfo_descriptors(bridge = True))
    self.assertEqual(6, len(descriptors))

    f = descriptors[0]
    self.assertEqual('BridgeExtraInfoDescriptor', type(f).__name__)
    self.assertEqual('A0187027648A392C6AC413B66F7CD25DD001BF76', f.fingerprint)

  @patch('stem.util.connection.download')
  @patch('stem.descriptor.collector.CollecTor.files')
  def test_reading_microdescriptors(self, files_mock, download_mock):
    with open(get_resource('collector/microdescs-2019-05-cropped.tar'), 'rb') as archive:
      download_mock.return_value = archive.read()

    files_mock.return_value = [stem.descriptor.collector.File(
      'archive/relay-descriptors/microdescs/microdescs-2014-01.tar',
      ['microdescriptor 1.0', 'network-status-microdesc-consensus-3 1.0'],
      7515396,
      'DFugbV1phhpiEB0QeyyueKp0V/bicmAAkdBk/95RjKk=',
      '2014-01-22 09:00',
      '2014-01-31 23:00',
      '2014-02-07 03:59',
    )]

    descriptors = list(stem.descriptor.collector.get_microdescriptors())
    self.assertEqual(3, len(descriptors))

    f = descriptors[0]
    self.assertEqual('Microdescriptor', type(f).__name__)
    self.assertEqual(['ed25519'], list(f.identifiers.keys()))

  @patch('stem.util.connection.download')
  @patch('stem.descriptor.collector.CollecTor.files')
  def test_reading_consensus(self, files_mock, download_mock):
    with open(get_resource('collector/consensuses-2018-06-cropped.tar'), 'rb') as archive:
      download_mock.return_value = archive.read()

    files_mock.return_value = [stem.descriptor.collector.File(
      'archive/relay-descriptors/consensuses/2019-11-27-23-00-00-consensus.tar',
      ['network-status-consensus-3 1.0'],
      2208505,
      'cGWT19Y0UVE/EUi3ZayacGvJU5t9T6MKaTOrNarAqlI=',
      '2019-11-27 23:00',
      '2019-11-27 23:00',
      '2019-11-27 23:05',
    )]

    descriptors = list(stem.descriptor.collector.get_consensus())
    self.assertEqual(243, len(descriptors))

    f = descriptors[0]
    self.assertEqual('RouterStatusEntryV3', type(f).__name__)
    self.assertEqual('000A10D43011EA4928A35F610405F92B4433B4DC', f.fingerprint)

    descriptors = list(stem.descriptor.collector.get_consensus(document_handler = DocumentHandler.DOCUMENT))
    self.assertEqual(2, len(descriptors))

    f = descriptors[0]
    self.assertEqual('NetworkStatusDocumentV3', type(f).__name__)
    self.assertEqual(35, len(f.routers))

    # this archive shouldn't have any v2 or microdescriptor consensus data

    self.assertEqual(0, len(list(stem.descriptor.collector.get_consensus(version = 2))))
    self.assertEqual(0, len(list(stem.descriptor.collector.get_consensus(microdescriptor = True))))

  @patch('stem.util.connection.download')
  @patch('stem.descriptor.collector.CollecTor.files')
  def test_reading_microdescriptor_consensus(self, files_mock, download_mock):
    with open(get_resource('collector/microdescs-2019-05-cropped.tar'), 'rb') as archive:
      download_mock.return_value = archive.read()

    files_mock.return_value = [stem.descriptor.collector.File(
      'archive/relay-descriptors/microdescs/microdescs-2014-01.tar',
      ['microdescriptor 1.0', 'network-status-microdesc-consensus-3 1.0'],
      7515396,
      'DFugbV1phhpiEB0QeyyueKp0V/bicmAAkdBk/95RjKk=',
      '2014-01-22 09:00',
      '2014-01-31 23:00',
      '2014-02-07 03:59',
    )]

    descriptors = list(stem.descriptor.collector.get_consensus(microdescriptor = True))
    self.assertEqual(556, len(descriptors))

    f = descriptors[0]
    self.assertEqual('RouterStatusEntryMicroV3', type(f).__name__)
    self.assertEqual('000A10D43011EA4928A35F610405F92B4433B4DC', f.fingerprint)

  @patch('stem.util.connection.download')
  @patch('stem.descriptor.collector.CollecTor.files')
  def test_reading_bridge_consensus(self, files_mock, download_mock):
    with open(get_resource('collector/bridge-statuses-2019-05-cropped.tar'), 'rb') as archive:
      download_mock.return_value = archive.read()

    files_mock.return_value = [stem.descriptor.collector.File(
      'archive/bridge-descriptors/microdescs/bridge-statuses-2008-05.tar',
      ['bridge-network-status 1.1'],
      74792,
      'scynC2b8xKD+NbkejGK7mKCegUwGPwgzXu7MouxBSj0=',
      '2008-05-16 19:46',
      '2008-05-31 23:37',
      '2016-09-14 21:11',
    )]

    descriptors = list(stem.descriptor.collector.get_consensus(bridge = True))
    self.assertEqual(2593, len(descriptors))

    f = descriptors[0]
    self.assertEqual('RouterStatusEntryBridgeV2', type(f).__name__)
    self.assertEqual('0035EA2A61E28D395F080ACA2244539490E70950', f.fingerprint)

  @patch('stem.util.connection.download')
  @patch('stem.descriptor.collector.CollecTor.files')
  def test_reading_key_certificates(self, files_mock, download_mock):
    with open(get_resource('collector/certs-cropped.tar'), 'rb') as archive:
      download_mock.return_value = archive.read()

    files_mock.return_value = [stem.descriptor.collector.File(
      'archive/relay-descriptors/certs.tar',
      ['dir-key-certificate-3 1.0'],
      151748,
      'ZfcE9RJwHvXhXaZ2xDzpoOJFqJeQR5ovOePOyNkKDi8=',
      '2007-09-19 03:14',
      '2019-10-08 04:06',
      '2019-11-29 03:33',
    )]

    descriptors = list(stem.descriptor.collector.get_key_certificates())
    self.assertEqual(5, len(descriptors))

    f = descriptors[0]
    self.assertEqual('KeyCertificate', type(f).__name__)
    self.assertEqual('14C131DFC5C6F93646BE72FA1401C02A8DF2E8B4', f.fingerprint)

  @patch('stem.util.connection.download')
  @patch('stem.descriptor.collector.CollecTor.files')
  def test_reading_bandwidth_files(self, files_mock, download_mock):
    with open(get_resource('collector/bandwidths-2019-05-cropped.tar'), 'rb') as archive:
      download_mock.return_value = archive.read()

    files_mock.return_value = [stem.descriptor.collector.File(
      'archive/relay-descriptors/bandwidths/bandwidths-2017-08.tar',
      ['bandwidth-file 1.0'],
      13330020,
      'BVVDEkegsLzkAn30dYikr4yTT79+XScfc0VUVEb83tM=',
      '2017-08-09 09:35',
      '2017-08-31 23:35',
      '2019-07-29 18:45',
    )]

    descriptors = list(stem.descriptor.collector.get_bandwidth_files())
    self.assertEqual(2, len(descriptors))

    f = descriptors[0]
    self.assertEqual('BandwidthFile', type(f).__name__)
    self.assertEqual(22, len(f.measurements))

  @patch('stem.util.connection.download')
  @patch('stem.descriptor.collector.CollecTor.files')
  def test_reading_exit_lists(self, files_mock, download_mock):
    with open(get_resource('collector/exit-list-2018-11-cropped.tar'), 'rb') as archive:
      download_mock.return_value = archive.read()

    files_mock.return_value = [stem.descriptor.collector.File(
      'archive/exit-lists/exit-list-2010-02.tar',
      ['tordnsel 1.0'],
      272008,
      'Q6ZAAy7RVbO+8rHH48AEZUU9PqcY5jD9zMASqjMzyns=',
      '2010-02-22 15:32',
      '2010-02-28 23:18',
      '2012-05-31 18:57',
    )]

    descriptors = list(stem.descriptor.collector.get_exit_lists())
    self.assertEqual(3713, len(descriptors))

    f = descriptors[0]
    self.assertEqual('TorDNSEL', type(f).__name__)
    self.assertEqual('0011BD2485AD45D984EC4159C88FC066E5E3300E', f.fingerprint)



"""
Unit tests for the stem.util.system functions. This works by mocking the
stem.util.system.call function to selectively exercise other functions. None of
these tests actually make system calls, use proc, or otherwise deal with the
system running the tests.
"""

import functools
import ntpath
import os
import posixpath
import tempfile
import unittest

import stem.prereq

from stem.util import system

try:
  # added in python 3.3
  from unittest.mock import Mock, patch
except ImportError:
  from mock import Mock, patch

# Base responses for the pid_by_name tests. The 'success' and
# 'multiple_results' entries are filled in by tests.

GET_PID_BY_NAME_BASE_RESULTS = {
  'success': [],
  'multiple_results': [],
  'malformed_data': ['bad data'],
  'no_results': [],
  'command_fails': None,
}

# testing output for system calls

GET_PID_BY_NAME_PS_BSD = [
  '  PID   TT  STAT      TIME COMMAND',
  '    1   ??  Ss     9:00.22 launchd',
  '   10   ??  Ss     0:09.97 kextd',
  '   11   ??  Ss     5:47.36 DirectoryService',
  '   12   ??  Ss     3:01.44 notifyd']

GET_PID_BY_NAME_PS_BSD_MULTIPLE = [
  '  PID   TT  STAT      TIME COMMAND',
  '    1   ??  Ss     9:00.22 launchd',
  '   10   ??  Ss     0:09.97 kextd',
  '   41   ??  Ss     9:00.22 launchd']


GET_PID_BY_NAME_TASKLIST_RESULTS = [
  'Image Name                     PID Session Name        Session#    Mem Usage',
  'System Idle Process              0 Services                   0         20 K',
  'svchost.exe                    872 Services                   0      8,744 K',
  'hpservice.exe                 1112 Services                   0      3,828 K',
  'tor.exe                       3712 Console                    1     29,976 K',
  'tor.exe                       3713 Console                    1     21,976 K',
  'conhost.exe                   3012 Console                    1      4,652 K']

GET_PID_BY_PORT_NETSTAT_RESULTS = [
  'Active Internet connections (only servers)',
  'Proto Recv-Q Send-Q Local Address           Foreign Address   State    PID/Program name',
  'tcp        0      0 127.0.0.1:631           0.0.0.0:*         LISTEN   -     ',
  'tcp        0      0 127.0.0.1:9051          0.0.0.0:*         LISTEN   1641/tor  ',
  'tcp6       0      0 ::1:631                 :::*              LISTEN   -     ',
  'udp        0      0 0.0.0.0:5353            0.0.0.0:*                  -     ',
  'udp6       0      0 fe80::7ae4:ff:fe2f::123 :::*                       -     ']

GET_PID_BY_PORT_SOCKSTAT_RESULTS = [
  '_tor     tor        4397  7  tcp4   51.64.7.84:9051    *:*',
  '_tor     tor        4397  12 tcp4   51.64.7.84:54011   80.3.121.7:9051',
  '_tor     tor        4397  15 tcp4   51.64.7.84:59374   7.42.1.102:9051']

GET_PID_BY_PORT_LSOF_RESULTS = [
  'COMMAND  PID   USER   FD   TYPE DEVICE SIZE/OFF NODE NAME',
  'tor     1745 atagar    6u  IPv4  14229      0t0  TCP 127.0.0.1:9051 (LISTEN)',
  'apache   329 atagar    6u  IPv4  14229      0t0  TCP 127.0.0.1:80 (LISTEN)']

GET_BSD_JAIL_PATH_RESULTS = [
  '   JID  IP Address      Hostname      Path',
  '     1  10.0.0.2        tor-jail      /usr/jails/tor-jail',
]


def mock_call(base_cmd, responses):
  """
  Provides mocking for the system module's call function. There are a couple
  ways of using this...

  - Simple usage is for base_cmd is the system call we want to respond to and
    responses is a list containing the respnose. For instance...

    mock_call('ls my_dir', ['file1', 'file2', 'file3'])

  - The base_cmd can be a formatted string and responses are a dictionary of
    completions for tat string to the responses. For instance...

    mock_call('ls %s', {'dir1': ['file1', 'file2'], 'dir2': ['file3', 'file4']})

  :param str base_cmd: command to match against
  :param list,dict responses: either list with the response, or mapping of
    base_cmd formatted string completions to responses

  :returns: **functor** to override stem.util.system.call with
  """

  def _mock_call(base_cmd, responses, command, default = None):
    if isinstance(responses, list):
      if base_cmd == command:
        return responses
      else:
        return default
    else:
      for cmd_completion in responses:
        if command == base_cmd % cmd_completion:
          return responses[cmd_completion]

      return default

  return functools.partial(_mock_call, base_cmd, responses)


class TestSystem(unittest.TestCase):
  @patch('stem.util.system.call')
  @patch('stem.util.system.is_available', Mock(return_value = True))
  def test_is_running(self, call_mock):
    """
    Exercises multiple use cases for the is_running function.
    """

    # mock response with a linux and bsd resolver
    running_commands = ['irssi', 'moc', 'tor', 'ps', '  firefox  ']

    for ps_cmd in (system.IS_RUNNING_PS_LINUX, system.IS_RUNNING_PS_BSD):
      call_mock.side_effect = mock_call(ps_cmd, running_commands)

      self.assertTrue(system.is_running('irssi'))
      self.assertTrue(system.is_running('moc'))
      self.assertTrue(system.is_running('tor'))
      self.assertTrue(system.is_running(['funky-tor', 'tor']))
      self.assertTrue(system.is_running('ps'))
      self.assertTrue(system.is_running('firefox'))
      self.assertFalse(system.is_running('something_else'))
      self.assertFalse(system.is_running(['funky-tor', 'funkier-tor']))

    # mock both calls failing

    call_mock.return_value = None
    call_mock.side_effect = None
    self.assertFalse(system.is_running('irssi'))
    self.assertEqual(None, system.is_running('irssi'))

  def test_size_of(self):
    """
    Exercises the size_of function.
    """

    if stem.prereq.is_pypy():
      self.assertRaises(NotImplementedError, system.size_of, 'hello')
    else:
      self.assertTrue(system.size_of('') < system.size_of('hello') < system.size_of('hello world'))
      self.assertTrue(system.size_of([]) < system.size_of(['hello']) < system.size_of(['hello', 'world']))
      self.assertTrue(system.size_of({}) < system.size_of({'hello': 'world'}) < system.size_of({'hello': 'world', 'more': 'stuff'}))

  @patch('stem.util.system.call')
  @patch('stem.util.proc.is_available', Mock(return_value = False))
  @patch('stem.util.system.is_available', Mock(return_value = True))
  def test_name_by_pid_ps(self, call_mock):
    """
    Tests the name_by_pid function with ps responses.
    """

    responses = {
      'success': ['COMMAND', 'vim'],
      'malformed_command_1': ['COMMAND'],
      'malformed_command_2': ['foobar'],
      'malformed_command_3': ['NOT_COMMAND', 'vim'],
      'no_results': [],
      'command_fails': None,
    }

    call_mock.side_effect = mock_call(system.GET_NAME_BY_PID_PS, responses)

    for test_input in responses:
      expected_response = 'vim' if test_input == 'success' else None
      self.assertEqual(expected_response, system.name_by_pid(test_input))

  @patch('stem.util.system.call')
  @patch('stem.util.system.is_available', Mock(return_value = True))
  @patch('stem.util.system.is_windows', Mock(return_value = False))
  def test_pid_by_name_pgrep(self, call_mock):
    """
    Tests the pid_by_name function with pgrep responses.
    """

    responses = dict(GET_PID_BY_NAME_BASE_RESULTS)
    responses['success'] = ['1111']
    responses['multiple_results'] = ['123', '456', '789']
    call_mock.side_effect = mock_call(system.GET_PID_BY_NAME_PGREP, responses)

    for test_input in responses:
      expected_response = 1111 if test_input == 'success' else None
      self.assertEqual(expected_response, system.pid_by_name(test_input))

    self.assertEqual([123, 456, 789], system.pid_by_name('multiple_results', multiple = True))

  @patch('stem.util.system.call')
  @patch('stem.util.system.is_available', Mock(return_value = True))
  @patch('stem.util.system.is_windows', Mock(return_value = False))
  def test_pid_by_name_pidof(self, call_mock):
    """
    Tests the pid_by_name function with pidof responses.
    """

    responses = dict(GET_PID_BY_NAME_BASE_RESULTS)
    responses['success'] = ['1111']
    responses['multiple_results'] = ['123 456 789']
    call_mock.side_effect = mock_call(system.GET_PID_BY_NAME_PIDOF, responses)

    for test_input in responses:
      expected_response = 1111 if test_input == 'success' else None
      self.assertEqual(expected_response, system.pid_by_name(test_input))

    self.assertEqual([123, 456, 789], system.pid_by_name('multiple_results', multiple = True))

  @patch('stem.util.system.call')
  @patch('stem.util.system.is_bsd', Mock(return_value = False))
  @patch('stem.util.system.is_windows', Mock(return_value = False))
  @patch('stem.util.system.is_available', Mock(return_value = True))
  def test_pid_by_name_ps_linux(self, call_mock):
    """
    Tests the pid_by_name function with the linux variant of ps.
    """

    responses = dict(GET_PID_BY_NAME_BASE_RESULTS)
    responses['success'] = ['PID', ' 1111']
    responses['multiple_results'] = ['PID', ' 123', ' 456', ' 789']
    call_mock.side_effect = mock_call(system.GET_PID_BY_NAME_PS_LINUX, responses)

    for test_input in responses:
      expected_response = 1111 if test_input == 'success' else None
      self.assertEqual(expected_response, system.pid_by_name(test_input))

    self.assertEqual([123, 456, 789], system.pid_by_name('multiple_results', multiple = True))

  @patch('stem.util.system.call')
  @patch('stem.util.system.is_bsd', Mock(return_value = True))
  @patch('stem.util.system.is_windows', Mock(return_value = False))
  @patch('stem.util.system.is_available', Mock(return_value = True))
  def test_pid_by_name_ps_bsd(self, call_mock):
    """
    Tests the pid_by_name function with the bsd variant of ps.
    """

    call_mock.side_effect = mock_call(system.GET_PID_BY_NAME_PS_BSD, GET_PID_BY_NAME_PS_BSD)
    self.assertEqual(1, system.pid_by_name('launchd'))
    self.assertEqual(11, system.pid_by_name('DirectoryService'))
    self.assertEqual(None, system.pid_by_name('blarg'))

    call_mock.side_effect = mock_call(system.GET_PID_BY_NAME_PS_BSD, GET_PID_BY_NAME_PS_BSD_MULTIPLE)

    self.assertEqual([1, 41], system.pid_by_name('launchd', multiple = True))

  @patch('stem.util.system.call')
  @patch('stem.util.system.is_available', Mock(return_value = True))
  @patch('stem.util.system.is_windows', Mock(return_value = False))
  def test_pid_by_name_lsof(self, call_mock):
    """
    Tests the pid_by_name function with lsof responses.
    """

    responses = dict(GET_PID_BY_NAME_BASE_RESULTS)
    responses['success'] = ['1111']
    responses['multiple_results'] = ['123', '456', '789']
    call_mock.side_effect = mock_call(system.GET_PID_BY_NAME_LSOF, responses)

    for test_input in responses:
      expected_response = 1111 if test_input == 'success' else None
      self.assertEqual(expected_response, system.pid_by_name(test_input))

    self.assertEqual([123, 456, 789], system.pid_by_name('multiple_results', multiple = True))

  @patch('stem.util.system.call')
  @patch('stem.util.system.is_available', Mock(return_value = True))
  @patch('stem.util.system.is_windows', Mock(return_value = True))
  def test_pid_by_name_tasklist(self, call_mock):
    """
    Tests the pid_by_name function with tasklist responses.
    """

    call_mock.side_effect = mock_call('tasklist', GET_PID_BY_NAME_TASKLIST_RESULTS)
    self.assertEqual(3712, system.pid_by_name('tor'))
    self.assertEqual(None, system.pid_by_name('DirectoryService'))
    self.assertEqual(None, system.pid_by_name('blarg'))
    self.assertEqual([3712, 3713], system.pid_by_name('tor', multiple = True))

  @patch('stem.util.system.call')
  @patch('stem.util.system.is_available', Mock(return_value = True))
  def test_pid_by_port_netstat(self, call_mock):
    """
    Tests the pid_by_port function with a netstat response.
    """

    call_mock.side_effect = mock_call(system.GET_PID_BY_PORT_NETSTAT, GET_PID_BY_PORT_NETSTAT_RESULTS)
    self.assertEqual(1641, system.pid_by_port(9051))
    self.assertEqual(1641, system.pid_by_port('9051'))
    self.assertEqual(None, system.pid_by_port(631))
    self.assertEqual(None, system.pid_by_port(123))

  @patch('stem.util.system.call')
  @patch('stem.util.system.is_available', Mock(return_value = True))
  def test_pid_by_port_sockstat(self, call_mock):
    """
    Tests the pid_by_port function with a sockstat response.
    """
    call_mock.side_effect = mock_call(system.GET_PID_BY_PORT_SOCKSTAT % 9051, GET_PID_BY_PORT_SOCKSTAT_RESULTS)
    self.assertEqual(4397, system.pid_by_port(9051))
    self.assertEqual(4397, system.pid_by_port('9051'))
    self.assertEqual(None, system.pid_by_port(123))

  @patch('stem.util.system.call')
  @patch('stem.util.system.is_available', Mock(return_value = True))
  def test_pid_by_port_lsof(self, call_mock):
    """
    Tests the pid_by_port function with a lsof response.
    """

    call_mock.side_effect = mock_call(system.GET_PID_BY_PORT_LSOF, GET_PID_BY_PORT_LSOF_RESULTS)
    self.assertEqual(1745, system.pid_by_port(9051))
    self.assertEqual(1745, system.pid_by_port('9051'))
    self.assertEqual(329, system.pid_by_port(80))
    self.assertEqual(None, system.pid_by_port(123))

  @patch('stem.util.system.call')
  @patch('stem.util.system.is_available', Mock(return_value = True))
  def test_pid_by_open_file_lsof(self, call_mock):
    """
    Tests the pid_by_open_file function with a lsof response.
    """

    lsof_query = system.GET_PID_BY_FILE_LSOF % '/tmp/foo'
    call_mock.side_effect = mock_call(lsof_query, ['4762'])
    self.assertEqual(4762, system.pid_by_open_file('/tmp/foo'))

    call_mock.return_value = []
    call_mock.side_effect = None
    self.assertEqual(None, system.pid_by_open_file('/tmp/somewhere_else'))

  @patch('stem.util.system.call')
  @patch('stem.util.proc.is_available', Mock(return_value = False))
  @patch('stem.util.system.is_available', Mock(return_value = True))
  def test_cwd_pwdx(self, call_mock):
    """
    Tests the cwd function with a pwdx response.
    """
    responses = {
      '3799': ['3799: /home/atagar'],
      '5839': ['5839: No such process'],
      '1234': ['malformed output'],
      '7878': None,
    }

    call_mock.side_effect = mock_call(system.GET_CWD_PWDX, responses)

    for test_input in responses:
      expected_response = '/home/atagar' if test_input == '3799' else None
      self.assertEqual(expected_response, system.cwd(test_input))

  @patch('stem.util.system.call')
  @patch('stem.util.proc.is_available', Mock(return_value = False))
  @patch('stem.util.system.is_available', Mock(return_value = True))
  def test_cwd_lsof(self, call_mock):
    """
    Tests the cwd function with a lsof response.
    """

    responses = {
      '75717': ['p75717', 'n/Users/atagar/tor/src/or'],
      '75718': ['p75718', 'fcwd', 'n/Users/atagar/tor/src/or'],
      '1234': ['malformed output'],
      '7878': [],
    }

    call_mock.side_effect = mock_call(system.GET_CWD_LSOF, responses)

    for test_input in responses:
      expected_response = '/Users/atagar/tor/src/or' if test_input in ('75717', '75718') else None
      self.assertEqual(expected_response, system.cwd(test_input))

  def test_tail(self):
    """
    Exercise our tail() function with a variety of inputs.
    """

    path = os.path.join(os.path.dirname(__file__), 'text_file')

    # by file handle

    with open(path, 'rb') as riddle_file:
      self.assertEqual(['  both the wicked and sweet.'], list(system.tail(riddle_file, 1)))

    self.assertEqual([], list(system.tail(path, 0)))
    self.assertEqual(['  both the wicked and sweet.'], list(system.tail(path, 1)))
    self.assertEqual(['  both the wicked and sweet.', "but I'm with people you meet"], list(system.tail(path, 2)))

    self.assertEqual(14, len(list(system.tail(path))))
    self.assertEqual(14, len(list(system.tail(path, 200))))

    self.assertRaises(IOError, list, system.tail('/path/doesnt/exist'))

    fd, temp_path = tempfile.mkstemp()
    os.chmod(temp_path, 0o077)  # remove read permissions
    # AssertionError: OSError not raised by list
    # self.assertRaises(IOError, list, system.tail(temp_path))
    os.close(fd)
    os.remove(temp_path)

  @patch('stem.util.system.call')
  @patch('stem.util.system.is_available', Mock(return_value = True))
  def test_bsd_jail_id(self, call_mock):
    """
    Tests the bsd_jail_id function.
    """

    responses = {
      '1111': ['JID', ' 1'],
      '2222': ['JID', ' 0'],
      '3333': ['JID', 'bad data'],
      '4444': ['bad data'],
      '5555': [],
      '6666': []
    }

    call_mock.side_effect = mock_call(system.GET_BSD_JAIL_ID_PS, responses)

    for test_input in responses:
      expected_response = 1 if test_input == '1111' else 0
      self.assertEqual(expected_response, system.bsd_jail_id(test_input))

  @patch('stem.util.system.call')
  @patch('stem.util.system.is_available', Mock(return_value = True))
  def test_bsd_jail_path(self, call_mock):
    """
    Tests the bsd_jail_path function.
    """

    # check when we don't have a jail

    call_mock.return_value = []
    self.assertEqual(None, system.bsd_jail_path(1))

    call_mock.side_effect = mock_call(system.GET_BSD_JAIL_PATH % '1', GET_BSD_JAIL_PATH_RESULTS)
    self.assertEqual('/usr/jails/tor-jail', system.bsd_jail_path(1))

  @patch('platform.system', Mock(return_value = 'Linux'))
  @patch('os.path.join', Mock(side_effect = posixpath.join))
  def test_expand_path_unix(self):
    """
    Tests the expand_path function. This does not exercise home directory
    expansions since that deals with our environment (that's left to integ
    tests).
    """

    self.assertEqual('', system.expand_path(''))
    self.assertEqual('/tmp', system.expand_path('/tmp'))
    self.assertEqual('/tmp', system.expand_path('/tmp/'))
    self.assertEqual('/tmp', system.expand_path('.', '/tmp'))
    self.assertEqual('/tmp', system.expand_path('./', '/tmp'))
    self.assertEqual('/tmp/foo', system.expand_path('foo', '/tmp'))
    self.assertEqual('/tmp/foo', system.expand_path('./foo', '/tmp'))

  @patch('platform.system', Mock(return_value = 'Windows'))
  @patch('os.path.join', Mock(side_effect = ntpath.join))
  def test_expand_path_windows(self):
    """
    Tests the expand_path function on windows. This does not exercise
    home directory expansions since that deals with our environment
    (that's left to integ tests).
    """

    self.assertEqual('', system.expand_path(''))
    self.assertEqual('C:\\tmp', system.expand_path('C:\\tmp'))
    self.assertEqual('C:\\tmp', system.expand_path('C:\\tmp\\'))
    self.assertEqual('C:\\tmp', system.expand_path('.', 'C:\\tmp'))
    self.assertEqual('C:\\tmp', system.expand_path('.\\', 'C:\\tmp'))
    self.assertEqual('C:\\tmp\\foo', system.expand_path('foo', 'C:\\tmp'))
    self.assertEqual('C:\\tmp\\foo', system.expand_path('.\\foo', 'C:\\tmp'))



"""
Unit tests for stem.descriptor.hidden_service for version 3.
"""

import base64
import functools
import unittest

import stem.client.datatype
import stem.descriptor
import stem.descriptor.hidden_service
import stem.prereq

import test.require

from stem.descriptor.hidden_service import (
  IntroductionPointV3,
  HiddenServiceDescriptorV3,
  AuthorizedClient,
  OuterLayer,
  InnerLayer,
)

from test.unit.descriptor import (
  get_resource,
  base_expect_invalid_attr,
  base_expect_invalid_attr_for_text,
)

try:
  # added in python 2.7
  from collections import OrderedDict
except ImportError:
  from stem.util.ordereddict import OrderedDict

try:
  # added in python 3.3
  from unittest.mock import patch, Mock
except ImportError:
  from mock import patch, Mock

require_sha3 = test.require.needs(stem.prereq._is_sha3_available, 'requires sha3')
require_x25519 = test.require.needs(lambda: stem.descriptor.hidden_service.X25519_AVAILABLE, 'requires openssl x5509')

expect_invalid_attr = functools.partial(base_expect_invalid_attr, HiddenServiceDescriptorV3, 'version', 3)
expect_invalid_attr_for_text = functools.partial(base_expect_invalid_attr_for_text, HiddenServiceDescriptorV3, 'version', 3)

HS_ADDRESS = u'sltib6sxkuxh2scmtuvd5w2g7pahnzkovefxpo4e4ptnkzl5kkq5h2ad.onion'
HS_PUBKEY = b'\x92\xe6\x80\xfaWU.}HL\x9d*>\xdbF\xfb\xc0v\xe5N\xa9\x0bw\xbb\x84\xe3\xe6\xd5e}R\xa1'

EXPECTED_SIGNING_CERT = """\
-----BEGIN ED25519 CERT-----
AQgABl5/AZLmgPpXVS59SEydKj7bRvvAduVOqQt3u4Tj5tVlfVKhAQAgBABUhpfe
/Wd3p/M74DphsGcIMee/npQ9BTzkzCyTyVmDbykek2EciWaOTCVZJVyiKPErngfW
BDwQZ8rhp05oCqhhY3oFHqG9KS7HGzv9g2v1/PrVJMbkfpwu1YK4b3zIZAk=
-----END ED25519 CERT-----\
"""

EXPECTED_OUTER_LAYER = """\
desc-auth-type foo
desc-auth-ephemeral-key bar
auth-client JNil86N07AA epkaL79NtajmgME/egi8oA qosYH4rXisxda3X7p9b6fw
auth-client 1D8VBAh9hdM 6K/uO3sRqBp6URrKC7GB6Q ElwRj5+6SN9kb8bRhiiQvA
encrypted
-----BEGIN MESSAGE-----
malformed block
-----END MESSAGE-----\
"""

with open(get_resource('hidden_service_v3')) as descriptor_file:
  HS_DESC_STR = descriptor_file.read()

with open(get_resource('hidden_service_v3_outer_layer')) as outer_layer_file:
  OUTER_LAYER_STR = outer_layer_file.read()

with open(get_resource('hidden_service_v3_inner_layer')) as inner_layer_file:
  INNER_LAYER_STR = inner_layer_file.read()

with open(get_resource('hidden_service_v3_intro_point')) as intro_point_file:
  INTRO_POINT_STR = intro_point_file.read()


class TestHiddenServiceDescriptorV3(unittest.TestCase):
  def test_real_descriptor(self):
    """
    Parse a descriptor for a testing hidden service from asn...

      sltib6sxkuxh2scmtuvd5w2g7pahnzkovefxpo4e4ptnkzl5kkq5h2ad.onion
    """

    with open(get_resource('hidden_service_v3'), 'rb') as descriptor_file:
      desc = next(stem.descriptor.parse_file(descriptor_file, 'hidden-service-descriptor-3 1.0', validate = True))

    self.assertEqual(3, desc.version)
    self.assertEqual(180, desc.lifetime)
    self.assertEqual(EXPECTED_SIGNING_CERT, str(desc.signing_cert))
    self.assertEqual(42, desc.revision_counter)
    self.assertTrue('eaH8VdaTKS' in desc.superencrypted)
    self.assertEqual('aglChCQF+lbzKgyxJJTpYGVShV/GMDRJ4+cRGCp+a2y/yX/tLSh7hzqI7rVZrUoGj74Xr1CLMYO3fXYCS+DPDQ', desc.signature)

  @require_sha3
  @test.require.ed25519_support
  def test_decryption(self):
    """
    Decrypt our descriptor and validate its content.
    """

    desc = HiddenServiceDescriptorV3.from_str(HS_DESC_STR)
    inner_layer = desc.decrypt(HS_ADDRESS)

    self.assertEqual(INNER_LAYER_STR, str(inner_layer))
    self.assertEqual(OUTER_LAYER_STR.rstrip('\x00'), str(inner_layer.outer))

  def test_outer_layer(self):
    """
    Parse the outer layer of our test descriptor.
    """

    desc = OuterLayer(OUTER_LAYER_STR)

    self.assertEqual('x25519', desc.auth_type)
    self.assertEqual('WjZCU9sV1oxkxaPcd7/YozeZgq0lEs6DhWyrdYRNJR4=', desc.ephemeral_key)
    self.assertTrue('BsRYMH/No+LgetIFv' in desc.encrypted)

    client = desc.clients['D0Bz0OlEMCg']

    self.assertEqual(16, len(desc.clients))
    self.assertEqual('D0Bz0OlEMCg', client.id)
    self.assertEqual('or3nS3ScSPYfLJuP9osGiQ', client.iv)
    self.assertEqual('B40RdIWhw7kdA7lt3KJPvQ', client.cookie)

  def test_inner_layer(self):
    """
    Parse the inner layer of our test descriptor.
    """

    desc = InnerLayer(INNER_LAYER_STR)

    self.assertEqual([2], desc.formats)
    self.assertEqual(['ed25519'], desc.intro_auth)
    self.assertEqual(True, desc.is_single_service)
    self.assertEqual(4, len(desc.introduction_points))

    intro_point = desc.introduction_points[0]

    self.assertEqual(2, len(intro_point.link_specifiers))

    link_specifier = intro_point.link_specifiers[0]
    self.assertEqual(stem.client.datatype.LinkByFingerprint, type(link_specifier))
    self.assertEqual('CCCCCCCCCCCCCCCCCCCC', link_specifier.fingerprint)

    link_specifier = intro_point.link_specifiers[1]
    self.assertEqual(stem.client.datatype.LinkByIPv4, type(link_specifier))
    self.assertEqual('1.2.3.4', link_specifier.address)
    self.assertEqual(9001, link_specifier.port)

    self.assertEqual('AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA=', intro_point.onion_key_raw)
    self.assertTrue('ID2l9EFNrp' in intro_point.auth_key_cert.to_base64())
    self.assertEqual('AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA=', intro_point.enc_key_raw)
    self.assertTrue('ZvjPt5IfeQ', intro_point.enc_key_cert)
    self.assertEqual(None, intro_point.legacy_key_raw)
    self.assertEqual(None, intro_point.legacy_key_cert)

  @test.require.ed25519_support
  def test_required_fields(self):
    """
    Check that we require the mandatory fields.
    """

    line_to_attr = {
      'hs-descriptor': 'version',
      'descriptor-lifetime': 'lifetime',
      'descriptor-signing-key-cert': 'signing_cert',
      'revision-counter': 'revision_counter',
      'superencrypted': 'superencrypted',
      'signature': 'signature',
    }

    for line in stem.descriptor.hidden_service.REQUIRED_V3_FIELDS:
      desc_text = HiddenServiceDescriptorV3.content(exclude = (line,))
      expect_invalid_attr_for_text(self, desc_text, line_to_attr[line], None)

  @test.require.ed25519_support
  def test_invalid_version(self):
    """
    Checks that our version field expects a numeric value.
    """

    test_values = (
      '',
      '-10',
      'hello',
    )

    for test_value in test_values:
      expect_invalid_attr(self, {'hs-descriptor': test_value}, 'version')

  @test.require.ed25519_support
  def test_invalid_lifetime(self):
    """
    Checks that our lifetime field expects a numeric value.
    """

    test_values = (
      '',
      '-10',
      'hello',
    )

    for test_value in test_values:
      expect_invalid_attr(self, {'descriptor-lifetime': test_value}, 'lifetime')

  @test.require.ed25519_support
  def test_invalid_revision_counter(self):
    """
    Checks that our revision counter field expects a numeric value.
    """

    test_values = (
      '',
      '-10',
      'hello',
    )

    for test_value in test_values:
      expect_invalid_attr(self, {'revision-counter': test_value}, 'revision_counter')

  @require_sha3
  def test_address_from_identity_key(self):
    self.assertEqual(HS_ADDRESS, HiddenServiceDescriptorV3.address_from_identity_key(HS_PUBKEY))

  @require_sha3
  def test_identity_key_from_address(self):
    self.assertEqual(HS_PUBKEY, HiddenServiceDescriptorV3.identity_key_from_address(HS_ADDRESS))
    self.assertRaisesWith(ValueError, "'boom.onion' isn't a valid hidden service v3 address", HiddenServiceDescriptorV3.identity_key_from_address, 'boom')
    self.assertRaisesWith(ValueError, 'Bad checksum (expected def7 but was 842e)', HiddenServiceDescriptorV3.identity_key_from_address, '5' * 56)

  def test_intro_point_parse(self):
    """
    Parse a v3 introduction point.
    """

    intro_point = IntroductionPointV3.parse(INTRO_POINT_STR)

    self.assertEqual('AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA=', intro_point.onion_key_raw)
    self.assertTrue('0Acq8QW8O7O' in intro_point.auth_key_cert.to_base64())
    self.assertEqual('AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA=', intro_point.enc_key_raw)
    self.assertTrue('4807i5', intro_point.enc_key_cert.to_base64())
    self.assertTrue('JAoGBAMO3' in intro_point.legacy_key_raw)
    self.assertTrue('Ln1ITJ0qP' in intro_point.legacy_key_cert)

  def test_intro_point_encode(self):
    """
    Encode an introduction point back into a string.
    """

    intro_point = IntroductionPointV3.parse(INTRO_POINT_STR)
    self.assertEqual(INTRO_POINT_STR.rstrip(), intro_point.encode())

  @require_x25519
  @test.require.ed25519_support
  def test_intro_point_crypto(self):
    """
    Retrieve IntroductionPointV3 cryptographic materials.
    """

    def base64_key(key):
      pubkey = stem.util._pubkey_bytes(key)
      pubkey_b64 = base64.b64encode(pubkey)
      return stem.util.str_tools._to_unicode(pubkey_b64)

    from cryptography.hazmat.primitives.asymmetric.x25519 import X25519PublicKey

    intro_point = InnerLayer(INNER_LAYER_STR).introduction_points[0]

    self.assertEqual('AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA=', intro_point.onion_key_raw)
    self.assertEqual('AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA=', intro_point.enc_key_raw)

    self.assertTrue(isinstance(intro_point.onion_key(), X25519PublicKey))
    self.assertTrue(isinstance(intro_point.enc_key(), X25519PublicKey))

    self.assertEqual(intro_point.onion_key_raw, base64_key(intro_point.onion_key()))
    self.assertEqual(intro_point.enc_key_raw, base64_key(intro_point.enc_key()))

    self.assertEqual(None, intro_point.legacy_key_raw)
    self.assertEqual(None, intro_point.legacy_key())

  @patch('stem.prereq.is_crypto_available', Mock(return_value = False))
  def test_intro_point_crypto_without_prereq(self):
    """
    Fetch cryptographic materials when the module is unavailable.
    """

    intro_point = InnerLayer(INNER_LAYER_STR).introduction_points[0]
    self.assertRaisesWith(ImportError, 'cryptography module unavailable', intro_point.onion_key)

  @test.require.ed25519_support
  def test_intro_point_creation(self):
    """
    Create an introduction point, encode it, then re-parse.
    """

    intro_point = IntroductionPointV3.create_for_address('1.1.1.1', 9001)

    self.assertEqual(1, len(intro_point.link_specifiers))
    self.assertEqual(stem.client.datatype.LinkByIPv4, type(intro_point.link_specifiers[0]))
    self.assertEqual('1.1.1.1', intro_point.link_specifiers[0].address)
    self.assertEqual(9001, intro_point.link_specifiers[0].port)

    reparsed = IntroductionPointV3.parse(intro_point.encode())
    self.assertEqual(intro_point, reparsed)

  @test.require.ed25519_support
  def test_inner_layer_creation(self):
    """
    Internal layer creation.
    """

    # minimal layer

    self.assertEqual(b'create2-formats 2', InnerLayer.content())
    self.assertEqual([2], InnerLayer.create().formats)

    # specify their only mandatory parameter (formats)

    self.assertEqual(b'create2-formats 1 2 3', InnerLayer.content({'create2-formats': '1 2 3'}))
    self.assertEqual([1, 2, 3], InnerLayer.create({'create2-formats': '1 2 3'}).formats)

    # include optional parameters

    desc = InnerLayer.create(OrderedDict((
      ('intro-auth-required', 'ed25519'),
      ('single-onion-service', ''),
    )))

    self.assertEqual([2], desc.formats)
    self.assertEqual(['ed25519'], desc.intro_auth)
    self.assertEqual(True, desc.is_single_service)
    self.assertEqual([], desc.introduction_points)

    # include introduction points

    desc = InnerLayer.create(introduction_points = [
      IntroductionPointV3.create_for_address('1.1.1.1', 9001),
      IntroductionPointV3.create_for_address('2.2.2.2', 9001),
      IntroductionPointV3.create_for_address('3.3.3.3', 9001),
    ])

    self.assertEqual(3, len(desc.introduction_points))
    self.assertEqual('1.1.1.1', desc.introduction_points[0].link_specifiers[0].address)

    self.assertTrue(InnerLayer.content(introduction_points = [
      IntroductionPointV3.create_for_address('1.1.1.1', 9001),
    ]).startswith(b'create2-formats 2\nintroduction-point AQAGAQEBASMp'))

  @test.require.ed25519_support
  def test_outer_layer_creation(self):
    """
    Outer layer creation.
    """

    from cryptography.hazmat.primitives.asymmetric.ed25519 import Ed25519PrivateKey

    # minimal layer

    self.assertTrue(OuterLayer.content().startswith(b'desc-auth-type x25519\ndesc-auth-ephemeral-key '))
    self.assertEqual('x25519', OuterLayer.create().auth_type)

    # specify the parameters

    desc = OuterLayer.create({
      'desc-auth-type': 'foo',
      'desc-auth-ephemeral-key': 'bar',
      'auth-client': [
        'JNil86N07AA epkaL79NtajmgME/egi8oA qosYH4rXisxda3X7p9b6fw',
        '1D8VBAh9hdM 6K/uO3sRqBp6URrKC7GB6Q ElwRj5+6SN9kb8bRhiiQvA',
      ],
      'encrypted': '\n-----BEGIN MESSAGE-----\nmalformed block\n-----END MESSAGE-----',
    })

    self.assertEqual('foo', desc.auth_type)
    self.assertEqual('bar', desc.ephemeral_key)
    self.assertEqual('-----BEGIN MESSAGE-----\nmalformed block\n-----END MESSAGE-----', desc.encrypted)

    self.assertEqual({
      '1D8VBAh9hdM': AuthorizedClient(id = b'1D8VBAh9hdM', iv = b'6K/uO3sRqBp6URrKC7GB6Q', cookie = b'ElwRj5+6SN9kb8bRhiiQvA'),
      'JNil86N07AA': AuthorizedClient(id = b'JNil86N07AA', iv = b'epkaL79NtajmgME/egi8oA', cookie = b'qosYH4rXisxda3X7p9b6fw'),
    }, desc.clients)

    self.assertEqual(EXPECTED_OUTER_LAYER, str(desc))

    # create an inner layer then decrypt it

    revision_counter = 5
    blinded_key = stem.util._pubkey_bytes(Ed25519PrivateKey.generate())
    subcredential = HiddenServiceDescriptorV3._subcredential(Ed25519PrivateKey.generate(), blinded_key)

    outer_layer = OuterLayer.create(
      inner_layer = InnerLayer.create(
        introduction_points = [
          IntroductionPointV3.create_for_address('1.1.1.1', 9001),
        ]
      ),
      revision_counter = revision_counter,
      subcredential = subcredential,
      blinded_key = blinded_key,
    )

    inner_layer = InnerLayer._decrypt(outer_layer, revision_counter, subcredential, blinded_key)

    self.assertEqual(1, len(inner_layer.introduction_points))
    self.assertEqual('1.1.1.1', inner_layer.introduction_points[0].link_specifiers[0].address)

  @test.require.ed25519_support
  def test_descriptor_creation(self):
    """
    HiddenServiceDescriptorV3 creation.
    """

    # minimal descriptor

    self.assertTrue(HiddenServiceDescriptorV3.content().startswith(b'hs-descriptor 3\ndescriptor-lifetime 180\n'))
    self.assertEqual(180, HiddenServiceDescriptorV3.create().lifetime)

    # specify the parameters

    desc = HiddenServiceDescriptorV3.create({
      'hs-descriptor': '4',
      'descriptor-lifetime': '123',
      'descriptor-signing-key-cert': '\n-----BEGIN ED25519 CERT-----\nmalformed block\n-----END ED25519 CERT-----',
      'revision-counter': '5',
      'superencrypted': '\n-----BEGIN MESSAGE-----\nmalformed block\n-----END MESSAGE-----',
      'signature': 'abcde',
    }, validate = False)

    self.assertEqual(4, desc.version)
    self.assertEqual(123, desc.lifetime)
    self.assertEqual(None, desc.signing_cert)  # malformed cert dropped because validation is disabled
    self.assertEqual(5, desc.revision_counter)
    self.assertEqual('-----BEGIN MESSAGE-----\nmalformed block\n-----END MESSAGE-----', desc.superencrypted)
    self.assertEqual('abcde', desc.signature)

    # include introduction points

    from cryptography.hazmat.primitives.asymmetric.ed25519 import Ed25519PrivateKey

    identity_key = Ed25519PrivateKey.generate()
    onion_address = HiddenServiceDescriptorV3.address_from_identity_key(identity_key)

    desc = HiddenServiceDescriptorV3.create(
      identity_key = identity_key,
      inner_layer = InnerLayer.create(introduction_points = [
        IntroductionPointV3.create_for_address('1.1.1.1', 9001),
        IntroductionPointV3.create_for_address('2.2.2.2', 9001),
        IntroductionPointV3.create_for_address('3.3.3.3', 9001),
      ]),
    )

    inner_layer = desc.decrypt(onion_address)
    self.assertEqual(3, len(inner_layer.introduction_points))
    self.assertEqual('1.1.1.1', inner_layer.introduction_points[0].link_specifiers[0].address)

  @test.require.ed25519_support
  def test_blinding(self):
    """
    Create a descriptor with key blinding.
    """

    from cryptography.hazmat.primitives.asymmetric.ed25519 import Ed25519PrivateKey

    expected_blinded_key = b'\xb5\xefEA\xfaI\x1a\xd8*p\xcd\x97\x01\x90O\xa8p\xd3\x10\x16\x8e-\x19\xab+\x92\xbc\xf6\xe7\x92\xc2k'

    desc = HiddenServiceDescriptorV3.create(
      identity_key = Ed25519PrivateKey.from_private_bytes(b'a' * 32),
      blinding_nonce = b'a' * 32,
    )

    self.assertEqual(64, len(desc.signing_cert.signature))
    self.assertEqual(expected_blinded_key, desc.signing_cert.signing_key())



from stem.descriptor.server_descriptor import RelayDescriptor

# prints 'caerSidi (71.35.133.197:9001)'
desc = RelayDescriptor.create()
print("%s (%s:%s)" % (desc.nickname, desc.address, desc.or_port))

# prints 'demo (127.0.0.1:80)'
desc = RelayDescriptor.create({'router': 'demo 127.0.0.1 80 0 0'})
print("%s (%s:%s)" % (desc.nickname, desc.address, desc.or_port))



from stem.descriptor import parse_file

server_descriptors = parse_file('/tmp/descriptor_dump', descriptor_type = 'server-descriptor 1.0')

for relay in server_descriptors:
  print(relay.fingerprint)



"""
Unit tests for the stem.version.Version parsing and class.
"""

import unittest

import stem.util.system
import stem.version

from stem.version import Version

try:
  # added in python 3.3
  from unittest.mock import Mock, patch
except ImportError:
  from mock import Mock, patch

VERSION_CMD_OUTPUT = """Mar 22 23:09:37.088 [notice] Tor v0.2.2.35 \
(git-73ff13ab3cc9570d). This is experimental software. Do not rely on it for \
strong anonymity. (Running on Linux i686)
%s"""

TOR_VERSION_OUTPUT = VERSION_CMD_OUTPUT % 'Tor version 0.2.2.35 (git-73ff13ab3cc9570d).'
MALFORMED_TOR_VERSION = VERSION_CMD_OUTPUT % 'Tor version 0.2.blah (git-73ff13ab3cc9570d).'
MISSING_TOR_VERSION = VERSION_CMD_OUTPUT % ''

TOR_VERSION_EXTRA_LINES = VERSION_CMD_OUTPUT % """
This is an extra line before the version
Tor version 0.2.2.35 (git-73ff13ab3cc9570d).
And an extra line afterward too
"""


class TestVersion(unittest.TestCase):
  @patch('stem.util.system.call')
  @patch.dict(stem.version.VERSION_CACHE)
  def test_get_system_tor_version(self, call_mock):
    call_mock.return_value = TOR_VERSION_OUTPUT.splitlines()

    version = stem.version.get_system_tor_version('tor_unit')

    self.assert_versions_match(version, 0, 2, 2, 35, None, 'git-73ff13ab3cc9570d')
    self.assertEqual('73ff13ab3cc9570d', version.git_commit)
    call_mock.assert_called_once_with('tor_unit --version')

    self.assertEqual(stem.version.VERSION_CACHE['tor_unit'], version)

  @patch('stem.util.system.call', Mock(return_value = TOR_VERSION_EXTRA_LINES.splitlines()))
  @patch.dict(stem.version.VERSION_CACHE)
  def test_get_system_tor_version_extra_lines(self):
    """
    Include extra text before and after the version.
    """

    version = stem.version.get_system_tor_version('tor_unit')
    self.assert_versions_match(version, 0, 2, 2, 35, None, 'git-73ff13ab3cc9570d')

  @patch('stem.util.system.call', Mock(return_value = MISSING_TOR_VERSION.splitlines()))
  @patch.dict(stem.version.VERSION_CACHE)
  def test_get_system_tor_version_missing(self):
    """
    Tor version output that doesn't include a version within it.
    """

    self.assertRaisesRegexp(IOError, "'tor_unit --version' didn't provide a parseable version", stem.version.get_system_tor_version, 'tor_unit')

  @patch('stem.util.system.call', Mock(return_value = MALFORMED_TOR_VERSION.splitlines()))
  @patch.dict(stem.version.VERSION_CACHE)
  def test_get_system_tor_version_malformed(self):
    """
    Tor version output that has the correct basic formatting, but an invalid
    version.
    """

    self.assertRaisesWith(IOError, "'0.2.blah (git-73ff13ab3cc9570d)' isn't a properly formatted tor version", stem.version.get_system_tor_version, 'tor_unit')

  def test_parsing(self):
    """
    Tests parsing by the Version class constructor.
    """

    # valid versions with various number of compontents to the version

    version = Version('0.1.2.3-tag')
    self.assert_versions_match(version, 0, 1, 2, 3, 'tag', None)

    version = Version('0.1.2.3')
    self.assert_versions_match(version, 0, 1, 2, 3, None, None)

    version = Version('0.1.2-tag')
    self.assert_versions_match(version, 0, 1, 2, None, 'tag', None)

    version = Version('0.1.2')
    self.assert_versions_match(version, 0, 1, 2, None, None, None)

    # checks an empty tag
    version = Version('0.1.2.3-')
    self.assert_versions_match(version, 0, 1, 2, 3, '', None)

    version = Version('0.1.2-')
    self.assert_versions_match(version, 0, 1, 2, None, '', None)

    # check with extra informaton
    version = Version('0.1.2.3-tag (git-73ff13ab3cc9570d)')
    self.assert_versions_match(version, 0, 1, 2, 3, 'tag', 'git-73ff13ab3cc9570d')
    self.assertEqual('73ff13ab3cc9570d', version.git_commit)

    version = Version('0.1.2.3-tag ()')
    self.assert_versions_match(version, 0, 1, 2, 3, 'tag', '')

    version = Version('0.1.2 (git-73ff13ab3cc9570d)')
    self.assert_versions_match(version, 0, 1, 2, None, None, 'git-73ff13ab3cc9570d')

    # checks invalid version strings
    self.assertRaises(ValueError, stem.version.Version, '')
    self.assertRaises(ValueError, stem.version.Version, '1.2.3.4nodash')
    self.assertRaises(ValueError, stem.version.Version, '1.2.3.a')
    self.assertRaises(ValueError, stem.version.Version, '1.2.a.4')
    self.assertRaises(ValueError, stem.version.Version, '1x2x3x4')
    self.assertRaises(ValueError, stem.version.Version, '12.3')
    self.assertRaises(ValueError, stem.version.Version, '1.-2.3')

  def test_with_multiple_extra(self):
    """
    Parse a version with multiple 'extra' fields.
    """

    version = Version('0.1.2 (release) (git-73ff13ab3cc9570d)')
    self.assert_versions_match(version, 0, 1, 2, None, None, 'release')
    self.assertEqual(['release', 'git-73ff13ab3cc9570d'], version.all_extra)
    self.assertEqual('73ff13ab3cc9570d', version.git_commit)

  def test_comparison(self):
    """
    Tests comparision between Version instances.
    """

    # check for basic incrementing in each portion
    self.assert_version_is_greater('1.1.2.3-tag', '0.1.2.3-tag')
    self.assert_version_is_greater('0.2.2.3-tag', '0.1.2.3-tag')
    self.assert_version_is_greater('0.1.3.3-tag', '0.1.2.3-tag')
    self.assert_version_is_greater('0.1.2.4-tag', '0.1.2.3-tag')
    self.assert_version_is_greater('0.1.2.3-ugg', '0.1.2.3-tag')
    self.assert_version_is_equal('0.1.2.3-tag', '0.1.2.3-tag')

    # check with common tags
    self.assert_version_is_greater('0.1.2.3-beta', '0.1.2.3-alpha')
    self.assert_version_is_greater('0.1.2.3-rc', '0.1.2.3-beta')

    # checks that a missing patch level equals zero
    self.assert_version_is_equal('0.1.2', '0.1.2.0')
    self.assert_version_is_equal('0.1.2-tag', '0.1.2.0-tag')

    # checks for missing patch or status
    self.assert_version_is_greater('0.1.2.3-tag', '0.1.2.3')
    self.assert_version_is_greater('0.1.2.3-tag', '0.1.2-tag')
    self.assert_version_is_greater('0.1.2.3-tag', '0.1.2')

    self.assert_version_is_equal('0.1.2.3', '0.1.2.3')
    self.assert_version_is_equal('0.1.2', '0.1.2')

  def test_nonversion_comparison(self):
    """
    Checks that we can be compared with other types.

    In python 3 on only equality comparisons work, greater than and less than
    comparisons result in a TypeError.
    """

    test_version = Version('0.1.2.3')
    self.assertNotEqual(test_version, None)
    self.assertNotEqual(test_version, 5)

  def test_string(self):
    """
    Tests the Version -> string conversion.
    """

    # checks conversion with various numbers of arguments
    self.assert_string_matches('0.1.2.3-tag')
    self.assert_string_matches('0.1.2.3')
    self.assert_string_matches('0.1.2')

  def test_requirements_greater_than(self):
    """
    Checks a VersionRequirements with a single greater_than rule.
    """

    requirements = stem.version._VersionRequirements()
    requirements.greater_than(Version('0.2.2.36'))

    self.assertTrue(Version('0.2.2.36') >= requirements)
    self.assertTrue(Version('0.2.2.37') >= requirements)
    self.assertTrue(Version('0.2.3.36') >= requirements)
    self.assertFalse(Version('0.2.2.35') >= requirements)
    self.assertFalse(Version('0.2.1.38') >= requirements)

    requirements = stem.version._VersionRequirements()
    requirements.greater_than(Version('0.2.2.36'), False)

    self.assertFalse(Version('0.2.2.35') >= requirements)
    self.assertFalse(Version('0.2.2.36') >= requirements)
    self.assertTrue(Version('0.2.2.37') >= requirements)

  def test_requirements_less_than(self):
    """
    Checks a VersionRequirements with a single less_than rule.
    """

    requirements = stem.version._VersionRequirements()
    requirements.less_than(Version('0.2.2.36'))

    self.assertTrue(Version('0.2.2.36') >= requirements)
    self.assertTrue(Version('0.2.2.35') >= requirements)
    self.assertTrue(Version('0.2.1.38') >= requirements)
    self.assertFalse(Version('0.2.2.37') >= requirements)
    self.assertFalse(Version('0.2.3.36') >= requirements)

    requirements = stem.version._VersionRequirements()
    requirements.less_than(Version('0.2.2.36'), False)

    self.assertFalse(Version('0.2.2.37') >= requirements)
    self.assertFalse(Version('0.2.2.36') >= requirements)
    self.assertTrue(Version('0.2.2.35') >= requirements)

  def test_requirements_in_range(self):
    """
    Checks a VersionRequirements with a single in_range rule.
    """

    requirements = stem.version._VersionRequirements()
    requirements.in_range(Version('0.2.2.36'), Version('0.2.2.38'))

    self.assertFalse(Version('0.2.2.35') >= requirements)
    self.assertTrue(Version('0.2.2.36') >= requirements)
    self.assertTrue(Version('0.2.2.37') >= requirements)
    self.assertFalse(Version('0.2.2.38') >= requirements)

    # rule for 'anything in the 0.2.2.x series'
    requirements = stem.version._VersionRequirements()
    requirements.in_range(Version('0.2.2.0'), Version('0.2.3.0'))

    for index in range(0, 100):
      self.assertTrue(Version('0.2.2.%i' % index) >= requirements)

  def test_requirements_multiple_rules(self):
    """
    Checks a VersionRequirements is the logical 'or' when it has multiple rules.
    """

    # rule to say 'anything but the 0.2.2.x series'
    requirements = stem.version._VersionRequirements()
    requirements.greater_than(Version('0.2.3.0'))
    requirements.less_than(Version('0.2.2.0'), False)

    self.assertTrue(Version('0.2.3.0') >= requirements)
    self.assertFalse(Version('0.2.2.0') >= requirements)

    for index in range(0, 100):
      self.assertFalse(Version('0.2.2.%i' % index) >= requirements)

  def assert_versions_match(self, version, major, minor, micro, patch, status, extra):
    """
    Asserts that the values for a types.Version instance match the given
    values.
    """

    self.assertEqual(major, version.major)
    self.assertEqual(minor, version.minor)
    self.assertEqual(micro, version.micro)
    self.assertEqual(patch, version.patch)
    self.assertEqual(status, version.status)
    self.assertEqual(extra, version.extra)

    if extra is None:
      self.assertEqual([], version.all_extra)
      self.assertEqual(None, version.git_commit)

  def assert_version_is_greater(self, first_version, second_version):
    """
    Asserts that the parsed version of the first version is greate than the
    second (also checking the inverse).
    """

    version1 = Version(first_version)
    version2 = Version(second_version)
    self.assertEqual(version1 > version2, True)
    self.assertEqual(version1 < version2, False)

  def assert_version_is_equal(self, first_version, second_version):
    """
    Asserts that the parsed version of the first version equals the second.
    """

    version1 = Version(first_version)
    version2 = Version(second_version)
    self.assertEqual(version1, version2)

  def assert_string_matches(self, version):
    """
    Parses the given version string then checks that its string representation
    matches the input.
    """

    self.assertEqual(version, str(Version(version)))



"""
Unit tests for stem.descriptor.microdescriptor.
"""

import unittest

import stem.descriptor
import stem.exit_policy
import test.require

from stem.descriptor.microdescriptor import Microdescriptor
from test.unit.descriptor import get_resource

FIRST_ONION_KEY = """\
-----BEGIN RSA PUBLIC KEY-----
MIGJAoGBAMhPQtZPaxP3ukybV5LfofKQr20/ljpRk0e9IlGWWMSTkfVvBcHsa6IM
H2KE6s4uuPHp7FqhakXAzJbODobnPHY8l1E4efyrqMQZXEQk2IMhgSNtG6YqUrVF
CxdSKSSy0mmcBe2TOyQsahlGZ9Pudxfnrey7KcfqnArEOqNH09RpAgMBAAE=
-----END RSA PUBLIC KEY-----\
"""

SECOND_ONION_KEY = """\
-----BEGIN RSA PUBLIC KEY-----
MIGJAoGBALCOxZdpMI2WO496njSQ2M7b4IgAGATqpJmH3So7lXOa25sK6o7JipgP
qQE83K/t/xsMIpxQ/hHkft3G78HkeXXFc9lVUzH0HmHwYEu0M+PMVULSkG36MfEl
7WeSZzaG+Tlnh9OySAzVyTsv1ZJsTQFHH9V8wuM0GOMo9X8DFC+NAgMBAAE=
-----END RSA PUBLIC KEY-----\
"""

THIRD_ONION_KEY = """\
-----BEGIN RSA PUBLIC KEY-----
MIGJAoGBAOWFQHxO+5kGuhwPUX5jB7wJCrTbSU0fZwolNV1t9UaDdjGDvIjIhdit
y2sMbyd9K8lbQO7x9rQjNst5ZicuaSOs854XQddSjm++vMdjYbOcVMqnKGSztvpd
w/1LVWFfhcBnsGi4JMGbmP+KUZG9A8kI9deSyJhfi35jA7UepiHHAgMBAAE=
-----END RSA PUBLIC KEY-----\
"""


class TestMicrodescriptor(unittest.TestCase):
  def test_from_str(self):
    sig = Microdescriptor.create()
    self.assertEqual(sig, Microdescriptor.from_str(str(sig)))

  def test_local_microdescriptors(self):
    """
    Checks a small microdescriptor file with known contents.
    """

    descriptor_path = get_resource('cached-microdescs')

    with open(descriptor_path, 'rb') as descriptor_file:
      descriptors = stem.descriptor.parse_file(descriptor_file, 'microdescriptor 1.0')

      router = next(descriptors)
      self.assertEqual(FIRST_ONION_KEY, router.onion_key)
      self.assertEqual(None, router.ntor_onion_key)
      self.assertEqual([], router.or_addresses)
      self.assertEqual([], router.family)
      self.assertEqual(stem.exit_policy.MicroExitPolicy('reject 1-65535'), router.exit_policy)
      self.assertEqual({b'@last-listed': b'2013-02-24 00:18:36'}, router.get_annotations())
      self.assertEqual([b'@last-listed 2013-02-24 00:18:36'], router.get_annotation_lines())

      router = next(descriptors)
      self.assertEqual(SECOND_ONION_KEY, router.onion_key)
      self.assertEqual('r5572HzD+PMPBbXlZwBhsm6YEbxnYgis8vhZ1jmdI2k=', router.ntor_onion_key)
      self.assertEqual([], router.or_addresses)
      self.assertEqual(['$6141629FA0D15A6AEAEF3A1BEB76E64C767B3174'], router.family)
      self.assertEqual(stem.exit_policy.MicroExitPolicy('reject 1-65535'), router.exit_policy)
      self.assertEqual({b'@last-listed': b'2013-02-24 00:18:37'}, router.get_annotations())
      self.assertEqual([b'@last-listed 2013-02-24 00:18:37'], router.get_annotation_lines())

      router = next(descriptors)
      self.assertEqual(THIRD_ONION_KEY, router.onion_key)
      self.assertEqual(None, router.ntor_onion_key)
      self.assertEqual([('2001:6b0:7:125::242', 9001, True)], router.or_addresses)
      self.assertEqual([], router.family)
      self.assertEqual(stem.exit_policy.MicroExitPolicy('accept 80,443'), router.exit_policy)
      self.assertEqual({b'@last-listed': b'2013-02-24 00:18:36'}, router.get_annotations())
      self.assertEqual([b'@last-listed 2013-02-24 00:18:36'], router.get_annotation_lines())

      self.assertEqual('uhCGfIM6RbeD1Z/C6e9ct41+NIl9EbpgP8wG7uZT2Rw', router.digest())
      self.assertEqual('@type microdescriptor 1.0', str(router.type_annotation()))

  def test_minimal_microdescriptor(self):
    """
    Basic sanity check that we can parse a microdescriptor with minimal
    attributes.
    """

    desc = Microdescriptor.create()

    self.assertEqual(None, desc.ntor_onion_key)
    self.assertEqual([], desc.or_addresses)
    self.assertEqual([], desc.family)
    self.assertEqual(stem.exit_policy.MicroExitPolicy('reject 1-65535'), desc.exit_policy)
    self.assertEqual(None, desc.exit_policy_v6)
    self.assertEqual({}, desc.identifiers)
    self.assertEqual(None, desc.identifier_type)
    self.assertEqual(None, desc.identifier)
    self.assertEqual({}, desc.protocols)
    self.assertEqual([], desc.get_unrecognized_lines())

  @test.require.cryptography
  def test_descriptor_signing(self):
    self.assertRaisesWith(NotImplementedError, 'Signing of Microdescriptor not implemented', Microdescriptor.create, sign = True)

  def test_unrecognized_line(self):
    """
    Includes unrecognized content in the descriptor.
    """

    desc = Microdescriptor.create({'pepperjack': 'is oh so tasty!'})
    self.assertEqual(['pepperjack is oh so tasty!'], desc.get_unrecognized_lines())

  def test_proceeding_line(self):
    """
    Includes a line prior to the 'onion-key' entry.
    """

    desc_text = b'family Amunet1\n' + Microdescriptor.content()
    self.assertRaises(ValueError, Microdescriptor, desc_text, True)

    desc = Microdescriptor(desc_text, validate = False)
    self.assertEqual(['Amunet1'], desc.family)

  def test_a_line(self):
    """
    Sanity test with both an IPv4 and IPv6 address.
    """

    desc_text = Microdescriptor.content()
    desc_text += b'\na 10.45.227.253:9001'
    desc_text += b'\na [fd9f:2e19:3bcf::02:9970]:9001'

    expected = [
      ('10.45.227.253', 9001, False),
      ('fd9f:2e19:3bcf::02:9970', 9001, True),
    ]

    desc = Microdescriptor(desc_text)
    self.assertEqual(expected, desc.or_addresses)

  def test_family(self):
    """
    Check the family line.
    """

    desc = Microdescriptor.create({'family': 'Amunet1 Amunet2 Amunet3'})
    self.assertEqual(['Amunet1', 'Amunet2', 'Amunet3'], desc.family)

    # try multiple family lines

    desc_text = Microdescriptor.content()
    desc_text += b'\nfamily Amunet1'
    desc_text += b'\nfamily Amunet2'

    self.assertRaises(ValueError, Microdescriptor, desc_text, True)

    # family entries will overwrite each other
    desc = Microdescriptor(desc_text, validate = False)
    self.assertEqual(1, len(desc.family))

  def test_exit_policy(self):
    """
    Basic check for 'p' lines. The router status entries contain an identical
    field so we're not investing much effort here.
    """

    desc = Microdescriptor.create({'p': 'accept 80,110,143,443'})
    self.assertEqual(stem.exit_policy.MicroExitPolicy('accept 80,110,143,443'), desc.exit_policy)

  def test_protocols(self):
    """
    Basic check for 'pr' lines.
    """

    desc = Microdescriptor.create({'pr': 'Cons=1 Desc=1 DirCache=1 HSDir=1 HSIntro=3 HSRend=1 Link=1-4 LinkAuth=1 Microdesc=1 Relay=1-2'})
    self.assertEqual(10, len(desc.protocols))

  def test_identifier(self):
    """
    Basic check for 'id' lines.
    """

    desc = Microdescriptor.create({'id': 'rsa1024 Cd47okjCHD83YGzThGBDptXs9Z4'})
    self.assertEqual({'rsa1024': 'Cd47okjCHD83YGzThGBDptXs9Z4'}, desc.identifiers)
    self.assertEqual('rsa1024', desc.identifier_type)
    self.assertEqual('Cd47okjCHD83YGzThGBDptXs9Z4', desc.identifier)

    # check when there's multiple key types

    desc_text = b'\n'.join((
      Microdescriptor.content(),
      b'id rsa1024 Cd47okjCHD83YGzThGBDptXs9Z4',
      b'id ed25519 50f6ddbecdc848dcc6b818b14d1',
    ))

    desc = Microdescriptor(desc_text, validate = True)
    self.assertEqual({'rsa1024': 'Cd47okjCHD83YGzThGBDptXs9Z4', 'ed25519': '50f6ddbecdc848dcc6b818b14d1'}, desc.identifiers)
    self.assertEqual('ed25519', desc.identifier_type)
    self.assertEqual('50f6ddbecdc848dcc6b818b14d1', desc.identifier)

    # check when there's conflicting keys

    desc_text = b'\n'.join((
      Microdescriptor.content(),
      b'id rsa1024 Cd47okjCHD83YGzThGBDptXs9Z4',
      b'id rsa1024 50f6ddbecdc848dcc6b818b14d1',
    ))

    desc = Microdescriptor(desc_text)
    self.assertEqual({}, desc.identifiers)

    exc_msg = "There can only be one 'id' line per a key type, but 'rsa1024' appeared multiple times"
    self.assertRaisesWith(ValueError, exc_msg, Microdescriptor, desc_text, validate = True)



# Copyright 2011-2019, Damian Johnson and The Tor Project
# See LICENSE for licensing information

"""
Unit and integration tests for the stem library. Helpers include...

::

  get_new_capabilities - missing capabilities found while testing
  register_new_capability - note that tor feature stem lacks

  get_all_combinations - provides all combinations of attributes
  tor_version - provides the version of tor we're testing against
"""

import collections
import itertools
import os

import stem.util.enum
import stem.version

__all__ = [
  'network',
  'output',
  'prompt',
  'runner',
]

# Integration targets fall into two categories:
#
# * Run Targets (like RUN_COOKIE and RUN_PTRACE) which customize our torrc.
#   We do an integration test run for each run target we get.
#
# * Attribute Target (like CHROOT and ONLINE) which indicates
#   non-configuration changes to your test runs. These are applied to all
#   integration runs that we perform.

Target = stem.util.enum.UppercaseEnum(
  'ONLINE',
  'RELATIVE',
  'CHROOT',
  'RUN_NONE',
  'RUN_OPEN',
  'RUN_PASSWORD',
  'RUN_COOKIE',
  'RUN_MULTIPLE',
  'RUN_SOCKET',
  'RUN_SCOOKIE',
  'RUN_PTRACE',
  'RUN_ALL',
)

AsyncTestArgs = collections.namedtuple('AsyncTestArgs', ['test_dir', 'tor_cmd'])
TOR_VERSION = None

# We make some paths relative to stem's base directory (the one above us)
# rather than the process' cwd. This doesn't end with a slash.

STEM_BASE = os.path.sep.join(__file__.split(os.path.sep)[:-2])

# Store new capabilities (events, descriptor entries, etc.)

NEW_CAPABILITIES = set()
NEW_CAPABILITIES_SUPPRESSION_TOKENS = set()

# File extensions of contents that should be ignored.

IGNORED_FILE_TYPES = []
GIT_IGNORE_PATH = os.path.join(STEM_BASE, '.gitignore')

if os.path.exists(GIT_IGNORE_PATH):
  with open(GIT_IGNORE_PATH) as ignore_file:
    for line in ignore_file:
      if line.startswith('*.'):
        IGNORED_FILE_TYPES.append(line[2:].strip())

if os.path.exists(os.path.join(STEM_BASE, '.travis.yml')):
    IGNORED_FILE_TYPES.append('.travis.yml')


def get_new_capabilities():
  """
  Provides a list of capabilities tor supports but stem doesn't, as discovered
  while running our tests.

  :returns: **set** of (type, message) tuples for the capabilities
  """

  return NEW_CAPABILITIES


def register_new_capability(capability_type, msg, suppression_token = None):
  """
  Register new capability found during the tests.

  :param str capability_type: type of capability this is
  :param str msg: description of what we found
  :param str suppression_token: skip registration if this token's already been
    provided
  """

  if suppression_token not in NEW_CAPABILITIES_SUPPRESSION_TOKENS:
    NEW_CAPABILITIES.add((capability_type, msg))

    if suppression_token:
      NEW_CAPABILITIES_SUPPRESSION_TOKENS.add(suppression_token)


def get_all_combinations(attr, include_empty = False):
  """
  Provides an iterator for all combinations of a set of attributes. For
  instance...

  ::

    >>> list(test.get_all_combinations(['a', 'b', 'c']))
    [('a',), ('b',), ('c',), ('a', 'b'), ('a', 'c'), ('b', 'c'), ('a', 'b', 'c')]

  :param list attr: attributes to provide combinations for
  :param bool include_empty: includes an entry with zero items if True
  :returns: iterator for all combinations
  """

  # Makes an itertools.product() call for 'i' copies of attr...
  #
  # * itertools.product(attr) => all one-element combinations
  # * itertools.product(attr, attr) => all two-element combinations
  # * ... etc

  if include_empty:
    yield ()

  seen = set()
  for index in range(1, len(attr) + 1):
    product_arg = [attr for _ in range(index)]

    for item in itertools.product(*product_arg):
      # deduplicate, sort, and only provide if we haven't seen it yet
      item = tuple(sorted(set(item)))

      if item not in seen:
        seen.add(item)
        yield item


def tor_version(tor_path = None):
  """
  Provides the version of tor we're testing against.

  :param str tor_path: location of tor executable to cehck the version of

  :returns: :class:`~stem.version.Version` of tor invoked by our integration
    tests
  """

  global TOR_VERSION

  if TOR_VERSION is None or tor_path:
    TOR_VERSION = stem.version.get_system_tor_version(tor_path)

  return TOR_VERSION



"""
Unit tests for the stem.Endpoint class.
"""

import unittest

import stem


class TestEndpoint(unittest.TestCase):
  def test_constructor(self):
    endpoint = stem.ORPort('12.34.56.78', 80)
    self.assertEqual('12.34.56.78', endpoint.address)
    self.assertEqual(80, endpoint.port)
    self.assertEqual(None, endpoint.link_protocols)

    endpoint = stem.ORPort('12.34.56.78', 80, [3])
    self.assertEqual('12.34.56.78', endpoint.address)
    self.assertEqual(80, endpoint.port)
    self.assertEqual([3], endpoint.link_protocols)

    endpoint = stem.DirPort('12.34.56.78', 80)
    self.assertEqual('12.34.56.78', endpoint.address)
    self.assertEqual(80, endpoint.port)

  def test_validation(self):
    self.assertRaises(ValueError, stem.DirPort, '12.34.56.78', 'hello')
    self.assertRaises(ValueError, stem.DirPort, '12.34.56.78', -5)
    self.assertRaises(ValueError, stem.DirPort, '12.34.56.78', None)
    self.assertRaises(ValueError, stem.DirPort, 'hello', 80)
    self.assertRaises(ValueError, stem.DirPort, -5, 80)
    self.assertRaises(ValueError, stem.DirPort, None, 80)

  def test_equality(self):
    self.assertTrue(stem.ORPort('12.34.56.78', 80) == stem.ORPort('12.34.56.78', 80))
    self.assertTrue(stem.ORPort('12.34.56.78', 80, [1, 2, 3]) == stem.ORPort('12.34.56.78', 80, [1, 2, 3]))
    self.assertFalse(stem.ORPort('12.34.56.78', 80) == stem.ORPort('12.34.56.88', 80))
    self.assertFalse(stem.ORPort('12.34.56.78', 80) == stem.ORPort('12.34.56.78', 443))
    self.assertFalse(stem.ORPort('12.34.56.78', 80, [2, 3]) == stem.ORPort('12.34.56.78', 80, [1, 2, 3]))

    self.assertTrue(stem.DirPort('12.34.56.78', 80) == stem.DirPort('12.34.56.78', 80))
    self.assertFalse(stem.DirPort('12.34.56.78', 80) == stem.DirPort('12.34.56.88', 80))
    self.assertFalse(stem.DirPort('12.34.56.78', 80) == stem.DirPort('12.34.56.78', 443))

    self.assertFalse(stem.ORPort('12.34.56.78', 80) == stem.DirPort('12.34.56.78', 80))
    self.assertFalse(stem.DirPort('12.34.56.78', 80) == stem.ORPort('12.34.56.78', 80))



"""
Unit tests for the stem.exit_policy.ExitPolicy class.
"""

import pickle
import unittest

try:
  # added in python 3.3
  from unittest.mock import Mock, patch
except ImportError:
  from mock import Mock, patch

from stem.exit_policy import (
  DEFAULT_POLICY_RULES,
  get_config_policy,
  ExitPolicy,
  MicroExitPolicy,
  ExitPolicyRule,
)


class TestExitPolicy(unittest.TestCase):
  def test_example(self):
    # tests the ExitPolicy and MicroExitPolicy pydoc examples
    policy = ExitPolicy('accept *:80', 'accept *:443', 'reject *:*')
    self.assertEqual('accept *:80, accept *:443, reject *:*', str(policy))
    self.assertEqual('accept 80, 443', policy.summary())
    self.assertTrue(policy.can_exit_to('75.119.206.243', 80))

    policy = MicroExitPolicy('accept 80,443')
    self.assertTrue(policy.can_exit_to('75.119.206.243', 80))

  def test_constructor(self):
    # The ExitPolicy constructor takes a series of string or ExitPolicyRule
    # entries. Extra whitespace is ignored to make csvs easier to handle.

    expected_policy = ExitPolicy(
      ExitPolicyRule('accept *:80'),
      ExitPolicyRule('accept *:443'),
      ExitPolicyRule('reject *:*'),
    )

    policy = ExitPolicy('accept *:80', 'accept *:443', 'reject *:*')
    self.assertEqual(expected_policy, policy)

    policy = ExitPolicy(*'accept *:80, accept *:443, reject *:*'.split(','))
    self.assertEqual(expected_policy, policy)

    # checks that we truncate after getting a catch-all policy

    policy = ExitPolicy(*'accept *:80, accept *:443, reject *:*, accept *:20-50'.split(','))
    self.assertEqual(expected_policy, policy)

    # checks that we compress redundant policies

    policy = ExitPolicy(*'reject *:80, reject *:443, reject *:*'.split(','))
    self.assertEqual(ExitPolicy('reject *:*'), policy)

  def test_can_exit_to(self):
    # Basic sanity test for our can_exit_to() method. Most of the interesting
    # use cases (ip masks, wildcards, etc) are covered by the ExitPolicyRule
    # tests.

    policy = ExitPolicy('accept *:80', 'accept *:443', 'reject *:*')

    for index in range(1, 100):
      ip_addr = '%i.%i.%i.%i' % (index / 2, index / 2, index / 2, index / 2)
      expected_result = index in (80, 443)

      self.assertEqual(expected_result, policy.can_exit_to(ip_addr, index))
      self.assertEqual(expected_result, policy.can_exit_to(port = index))

  def test_can_exit_to_strictness(self):
    # Check our 'strict' argument.

    policy = ExitPolicy('reject 1.0.0.0/8:80', 'accept *:*')
    self.assertEqual(False, policy.can_exit_to(None, 80, strict = True))  # can't exit to *all* instances of port 80
    self.assertEqual(True, policy.can_exit_to(None, 80, strict = False))  # can exit to *an* instance of port 80

    policy = ExitPolicy('accept 1.0.0.0/8:80', 'reject *:*')
    self.assertEqual(False, policy.can_exit_to(None, 80, strict = True))  # can't exit to *all* instances of port 80
    self.assertEqual(True, policy.can_exit_to(None, 80, strict = False))  # can exit to *an* instance of port 80

  def test_is_exiting_allowed(self):
    test_inputs = {
      (): True,
      ('accept *:*', ): True,
      ('reject *:*', ): False,
      ('accept *:80', 'reject *:*'): True,
      ('reject *:80', 'accept *:80', 'reject *:*'): False,
      ('reject *:50-90', 'accept *:80', 'reject *:*'): False,
      ('reject *:2-65535', 'accept *:80-65535', 'reject *:*'): False,
      ('reject *:2-65535', 'accept 127.0.0.0:1', 'reject *:*'): True,
      ('reject 127.0.0.1:*', 'accept *:80', 'reject *:*'): True,
    }

    for rules, expected_result in test_inputs.items():
      policy = ExitPolicy(*rules)
      self.assertEqual(expected_result, policy.is_exiting_allowed())

  def test_summary_examples(self):
    # checks the summary() method's pydoc examples

    policy = ExitPolicy('accept *:80', 'accept *:443', 'reject *:*')
    self.assertEqual('accept 80, 443', policy.summary())

    policy = ExitPolicy('accept *:443', 'reject *:1-1024', 'accept *:*')
    self.assertEqual('reject 1-442, 444-1024', policy.summary())

  def test_summary_large_ranges(self):
    # checks the summary() method when the policy includes very large port ranges

    policy = ExitPolicy('reject *:80-65535', 'accept *:1-65533', 'reject *:*')
    self.assertEqual('accept 1-79', policy.summary())

  def test_without_port(self):
    policy = get_config_policy('accept 216.58.193.78, reject *')
    self.assertEqual([ExitPolicyRule('accept 216.58.193.78:*'), ExitPolicyRule('reject *:*')], list(policy))

    policy = get_config_policy('reject6 [2a00:1450:4001:081e:0000:0000:0000:200e]')
    self.assertEqual([ExitPolicyRule('reject [2a00:1450:4001:081e:0000:0000:0000:200e]:*')], list(policy))

  def test_non_private_non_default_policy(self):
    policy = get_config_policy('reject *:80-65535, accept *:1-65533, reject *:*')

    for rule in policy:
      self.assertFalse(rule.is_private())
      self.assertFalse(rule.is_default())

    self.assertFalse(policy.has_private())
    self.assertFalse(policy.has_default())

    self.assertEqual(policy, policy.strip_private())
    self.assertEqual(policy, policy.strip_default())

  def test_all_private_policy(self):
    for port in ('*', '80', '1-1024'):
      private_policy = get_config_policy('reject private:%s' % port, '12.34.56.78')

      for rule in private_policy:
        self.assertTrue(rule.is_private())

      self.assertEqual(ExitPolicy(), private_policy.strip_private())

    # though not commonly done, technically private policies can be accept rules too

    private_policy = get_config_policy('accept private:*')
    self.assertEqual(ExitPolicy(), private_policy.strip_private())

  @patch('socket.gethostname', Mock(side_effect = IOError('no address')))
  def test_all_private_policy_without_network(self):
    for rule in get_config_policy('reject private:80, accept *:80'):
      # all rules except the ending accept are part of the private policy
      self.assertEqual(str(rule) != 'accept *:80', rule.is_private())

  def test_all_default_policy(self):
    policy = ExitPolicy(*DEFAULT_POLICY_RULES)

    for rule in policy:
      self.assertTrue(rule.is_default())

    self.assertTrue(policy.has_default())
    self.assertEqual(ExitPolicy(), policy.strip_default())

  def test_mixed_private_policy(self):
    policy = get_config_policy('accept *:80, reject private:1-65533, accept *:*')

    for rule in policy:
      self.assertTrue(rule.is_accept != rule.is_private())  # only reject rules are the private ones

    self.assertEqual(get_config_policy('accept *:80, accept *:*'), policy.strip_private())

  def test_mixed_default_policy(self):
    policy = ExitPolicy('accept *:80', 'accept 127.0.0.1:1-65533', *DEFAULT_POLICY_RULES)

    for rule in policy:
      # only accept-all and reject rules are the default ones
      self.assertTrue(rule.is_accept != rule.is_default() or (rule.is_accept and rule.is_address_wildcard() and rule.is_port_wildcard()))

    self.assertEqual(get_config_policy('accept *:80, accept 127.0.0.1:1-65533'), policy.strip_default())

  def test_get_config_policy_with_ipv6(self):
    # ensure our constructor accepts addresses both with and without brackets
    self.assertTrue(get_config_policy('reject private:80', 'fe80:0000:0000:0000:0202:b3ff:fe1e:8329').is_exiting_allowed())
    self.assertTrue(get_config_policy('reject private:80', '[fe80:0000:0000:0000:0202:b3ff:fe1e:8329]').is_exiting_allowed())

  def test_str(self):
    # sanity test for our __str__ method

    policy = ExitPolicy('  accept *:80\n', '\taccept *:443')
    self.assertEqual('accept *:80, accept *:443', str(policy))

    policy = ExitPolicy('reject 0.0.0.0/255.255.255.0:*', 'accept *:*')
    self.assertEqual('reject 0.0.0.0/24:*, accept *:*', str(policy))

  def test_iter(self):
    # sanity test for our __iter__ method

    rules = [
      ExitPolicyRule('accept *:80'),
      ExitPolicyRule('accept *:443'),
      ExitPolicyRule('reject *:*'),
    ]

    self.assertEqual(rules, list(ExitPolicy(*rules)))
    self.assertEqual(rules, list(ExitPolicy('accept *:80', 'accept *:443', 'reject *:*')))

  def test_microdescriptor_parsing(self):
    # mapping between inputs and if they should succeed or not
    test_inputs = {
      'accept 80': True,
      'accept 80,443': True,
      '': False,
      'accept': False,
      'accept ': False,
      'accept\t80,443': False,
      'accept 80, 443': False,
      'accept 80,\t443': False,
      '80,443': False,
      'accept 80,-443': False,
      'accept 80,+443': False,
      'accept 80,66666': False,
      'reject 80,foo': False,
      'bar 80,443': False,
    }

    for policy_arg, expect_success in test_inputs.items():
      try:
        policy = MicroExitPolicy(policy_arg)

        if expect_success:
          self.assertEqual(policy_arg, str(policy))
        else:
          self.fail()
      except ValueError:
        if expect_success:
          self.fail()

  def test_microdescriptor_attributes(self):
    # checks that its is_accept attribute is properly set

    # single port
    policy = MicroExitPolicy('accept 443')
    self.assertTrue(policy.is_accept)

    # multiple ports
    policy = MicroExitPolicy('accept 80,443')
    self.assertTrue(policy.is_accept)

    # port range
    policy = MicroExitPolicy('reject 1-1024')
    self.assertFalse(policy.is_accept)

  def test_microdescriptor_can_exit_to(self):
    test_inputs = {
      'accept 443': {442: False, 443: True, 444: False},
      'reject 443': {442: True, 443: False, 444: True},
      'accept 80,443': {80: True, 443: True, 10: False},
      'reject 1-1024': {1: False, 1024: False, 1025: True},
    }

    for policy_arg, attr in test_inputs.items():
      policy = MicroExitPolicy(policy_arg)

      for port, expected_value in attr.items():
        self.assertEqual(expected_value, policy.can_exit_to(port = port))

    # address argument should be ignored
    policy = MicroExitPolicy('accept 80,443')

    self.assertFalse(policy.can_exit_to('127.0.0.1', 79))
    self.assertTrue(policy.can_exit_to('127.0.0.1', 80))

  def test_get_config_policy(self):
    test_inputs = {
      '': ExitPolicy(),
      'reject *': ExitPolicy('reject *:*'),
      'reject *:*': ExitPolicy('reject *:*'),
      'reject private': ExitPolicy(
        'reject 0.0.0.0/8:*',
        'reject 169.254.0.0/16:*',
        'reject 127.0.0.0/8:*',
        'reject 192.168.0.0/16:*',
        'reject 10.0.0.0/8:*',
        'reject 172.16.0.0/12:*',
        'reject 12.34.56.78:*',
      ),
      'accept *:80, reject *': ExitPolicy(
        'accept *:80',
        'reject *:*',
      ),
      '  accept *:80,     reject *   ': ExitPolicy(
        'accept *:80',
        'reject *:*',
      ),
    }

    for test_input, expected in test_inputs.items():
      self.assertEqual(expected, get_config_policy(test_input, '12.34.56.78'))

    test_inputs = (
      'blarg',
      'accept *:*:*',
      'acceptt *:80',
      'accept 257.0.0.1:80',
      'accept *:999999',
    )

    for test_input in test_inputs:
      self.assertRaises(ValueError, get_config_policy, test_input)

  def test_pickleability(self):
    """
    Checks that we can unpickle ExitPolicy instances.
    """

    policy = ExitPolicy('accept *:80', 'accept *:443', 'reject *:*')
    self.assertTrue(policy.can_exit_to('74.125.28.106', 80))

    encoded_policy = pickle.dumps(policy)
    restored_policy = pickle.loads(encoded_policy)

    self.assertEqual(policy, restored_policy)
    self.assertTrue(restored_policy.is_exiting_allowed())
    self.assertTrue(restored_policy.can_exit_to('74.125.28.106', 80))



import json
import os
import re
import unittest

import test


class TestInstallation(unittest.TestCase):
  # TODO: remove when dropping support for python 2.6
  skip_reason = 'setUpClass() unsupported in python 2.6'

  @classmethod
  def setUpClass(self):
    setup_path = os.path.join(test.STEM_BASE, 'setup.py')
    self.skip_reason = None
    self.setup_contents = False

    if os.path.exists(setup_path):
      with open(setup_path) as setup_file:
        self.setup_contents = setup_file.read()
    else:
      self.skip_reason = '(only for git checkout)'

  def test_installs_all_modules(self):
    if self.skip_reason:
      self.skipTest(self.skip_reason)
      return True

    # Modules cited my our setup.py looks like...
    #
    #   packages = ['stem', 'stem.descriptor', 'stem.util'],

    modules = json.loads(re.search('packages = (\\[.*\\])', self.setup_contents).group(1).replace("'", '"'))
    module_paths = dict([(m, os.path.join(test.STEM_BASE, m.replace('.', os.path.sep))) for m in modules])

    for module, path in module_paths.items():
      if not os.path.exists(path):
        self.fail("setup.py's module %s doesn't exist at %s" % (module, path))

    for entry in os.walk(os.path.join(test.STEM_BASE, 'stem')):
      directory = entry[0]

      if directory.endswith('__pycache__'):
        continue
      elif directory not in module_paths.values():
        self.fail("setup.py doesn't install %s" % directory)

  def test_installs_all_data_files(self):
    if self.skip_reason:
      self.skipTest(self.skip_reason)
      return True

    # Checking that we have all non-source files. Data looks like...
    #
    #   package_data = {
    #     'stem': ['cached_fallbacks.cfg', 'cached_manual.cfg', 'settings.cfg'],
    #   },

    package_data = {}

    for line in self.setup_contents.split('package_data = {\n', 1)[1].splitlines():
      if '},' in line:
        break

      directory = line.strip().split()[0][1:-2]
      files = line.strip().split(' ', 1)[1][2:-3].split("', '")
      package_data[directory] = files

    data_files = []

    for module, files in package_data.items():
      for module_file in files:
        data_files.append(os.path.join(test.STEM_BASE, module.replace('.', os.path.sep), module_file))

    for path in data_files:
      if not os.path.exists(path):
        self.fail("setup.py installs a data file that doesn't exist: %s" % path)

    for entry in os.walk(os.path.join(test.STEM_BASE, 'stem')):
      directory = entry[0]

      if directory.endswith('__pycache__'):
        continue

      for filename in entry[2]:
        path = os.path.join(directory, filename)
        file_type = path.split('.')[-1]

        if file_type in (['py'] + test.IGNORED_FILE_TYPES):
          continue
        elif path not in data_files:
          self.fail("setup.py doesn't install %s" % path)



"""
Unit tests for the stem.util.enum class and functions.
"""

import unittest

import stem.util.enum


class TestEnum(unittest.TestCase):
  def test_enum_examples(self):
    """
    Checks that the pydoc examples are accurate.
    """

    insects = stem.util.enum.Enum('ANT', 'WASP', 'LADYBUG', 'FIREFLY')
    self.assertEqual('Ant', insects.ANT)
    self.assertEqual(('Ant', 'Wasp', 'Ladybug', 'Firefly'), tuple(insects))

    pets = stem.util.enum.Enum(('DOG', 'Skippy'), 'CAT', ('FISH', 'Nemo'))
    self.assertEqual('Skippy', pets.DOG)
    self.assertEqual('Cat', pets.CAT)

  def test_uppercase_enum_example(self):
    """
    Checks that the pydoc example for the UppercaseEnum constructor function is
    accurate.
    """

    runlevels = stem.util.enum.UppercaseEnum('DEBUG', 'INFO', 'NOTICE', 'WARN', 'ERROR')
    self.assertEqual('DEBUG', runlevels.DEBUG)

  def test_enum_methods(self):
    """
    Exercises enumeration methods.
    """

    insects = stem.util.enum.Enum('ANT', 'WASP', 'LADYBUG', 'FIREFLY')

    # next method
    self.assertEqual(insects.WASP, insects.next(insects.ANT))
    self.assertEqual(insects.ANT, insects.next(insects.FIREFLY))

    # previous method
    self.assertEqual(insects.FIREFLY, insects.previous(insects.ANT))
    self.assertEqual(insects.LADYBUG, insects.previous(insects.FIREFLY))

    # keys method
    self.assertEqual(['ANT', 'WASP', 'LADYBUG', 'FIREFLY'], list(insects.keys()))
